"""
对话记录标签页 - 展示所有对话内容
"""
import sys
import os
import json
import weakref
from typing import List, Dict, Optional
from PySide6.QtWidgets import (
    QWidget, QVBoxLayout, QHBoxLayout, QScrollArea, QFrame, QLabel, QSizePolicy, QPushButton
)
from PySide6.QtCore import Qt, QFile, QTextStream, QTimer
import pyperclip

try:
    from .base_tab import BaseTab
except ImportError:
    from base_tab import BaseTab

try:
    from ..components.markdown_display import MarkdownDisplayWidget
except ImportError:
    try:
        from components.markdown_display import MarkdownDisplayWidget
    except ImportError:
        from PySide6.QtWidgets import QTextEdit
        MarkdownDisplayWidget = QTextEdit


class ChatHistoryTab(BaseTab):
    """对话记录标签页 - 展示所有对话内容"""

    def __init__(self, project_path: Optional[str] = None, session_id: Optional[str] = None, workspace_id: Optional[str] = None, parent=None):
        super().__init__(parent)
        self.project_path = project_path
        self.session_id = session_id
        self.workspace_id = workspace_id

        # 计算 workspace_path
        if workspace_id and project_path:
            self.workspace_path = os.path.join(project_path, '.workspace', workspace_id)
        else:
            self.workspace_path = None

        # UI组件
        self.scroll_area = None
        self.messages_container = None
        self.messages_layout = None
        self.load_more_button = None

        # 历史记录管理
        self.all_history = []
        self.displayed_count = 10
        self.current_start_idx = -1  # 当前显示的起始索引，用于增量加载
        self._loaded = False  # 延迟加载标志

        self.create_ui()

    def create_ui(self):
        """创建对话记录Tab的UI"""
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)

        # 创建滚动区域
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        
        # 容器
        self.messages_container = QWidget()
        self.messages_container.setObjectName("messagesContainer")
        self.messages_layout = QVBoxLayout(self.messages_container)
        self.messages_layout.setContentsMargins(15, 15, 15, 15)
        self.messages_layout.setSpacing(5)
        self.messages_layout.setAlignment(Qt.AlignTop)

        self.scroll_area.setWidget(self.messages_container)
        layout.addWidget(self.scroll_area)

        # 加载样式表
        self._load_stylesheet()

    def _load_stylesheet(self):
        """加载QSS样式表"""
        try:
            current_dir = os.path.dirname(os.path.abspath(__file__))
            qss_path = os.path.join(current_dir, "chat_history_style.qss")
            qss_file = QFile(qss_path)
            if qss_file.open(QFile.ReadOnly | QFile.Text):
                stream = QTextStream(qss_file)
                self.setStyleSheet(stream.readAll())
                qss_file.close()
            else:
                print(f"无法加载样式表: {qss_path}", file=sys.stderr)
        except Exception as e:
            print(f"加载样式表出错: {e}", file=sys.stderr)

    def load_history(self):
        """加载并显示对话历史记录"""
        # 清空现有消息
        self._clear_messages()

        # 读取历史记录
        self.all_history = self._load_history_from_file()

        if not self.all_history:
            # 如果没有历史记录,显示提示
            self._show_empty_message()
            return

        # 显示最后5条记录
        self._display_records()

    def _display_records(self):
        """显示记录(从最新的开始显示指定数量)"""
        # 清空所有现有消息
        self._clear_messages()
        self.load_more_button = None

        total = len(self.all_history)
        # 计算要显示的记录范围
        start_idx = max(0, total - self.displayed_count)
        self.current_start_idx = start_idx
        records_to_show = self.all_history[start_idx:]

        # 如果还有更多记录,显示"加载更多"按钮
        if start_idx > 0:
            self._add_load_more_button()

        # 显示记录
        for record in records_to_show:
            self._render_record(record)

    def _render_record(self, record):
        """渲染单条记录"""
        role = record.get('role')
        if role == 'user':
            self._add_user_message(record.get('content', ''))
        elif role == 'assistant':
            self._add_assistant_message(record.get('content', ''))
        elif role == 'tool':
            name = record.get('name', '')
            # feedback 工具拆分为两条消息
            if 'feedback' in name.lower():
                self._add_feedback_messages(record)
            else:
                self._add_tool_message(
                    name,
                    record.get('input', {}),
                    record.get('output', ''),
                    record.get('timestamp', '')
                )

    def _add_load_more_button(self):
        """添加加载更多按钮"""
        self.load_more_button = QPushButton("点击查看更多")
        self.load_more_button.setObjectName("loadMoreButton")
        self.load_more_button.clicked.connect(self._load_more)
        self.load_more_button.setStyleSheet("""
            QPushButton {
                background-color: #3a3a3a;
                color: #e0e0e0;
                border: 1px solid #555;
                border-radius: 4px;
                padding: 8px 16px;
                font-size: 13px;
            }
            QPushButton:hover {
                background-color: #4a4a4a;
            }
        """)
        self.messages_layout.insertWidget(0, self.load_more_button)

    def _load_more(self):
        """增量加载更多记录，保持滚动位置"""
        if self.current_start_idx <= 0:
            return

        # 记录当前滚动位置
        scrollbar = self.scroll_area.verticalScrollBar()
        old_scroll_value = scrollbar.value()
        old_max = scrollbar.maximum()

        # 计算新的起始索引
        new_start_idx = max(0, self.current_start_idx - 10)
        new_records = self.all_history[new_start_idx:self.current_start_idx]
        self.current_start_idx = new_start_idx
        self.displayed_count += len(new_records)

        # 移除旧的"加载更多"按钮
        if self.load_more_button:
            self.messages_layout.removeWidget(self.load_more_button)
            self.load_more_button.deleteLater()
            self.load_more_button = None

        # 如果还有更多记录，添加新的"加载更多"按钮
        if new_start_idx > 0:
            self._add_load_more_button()

        # 记录插入位置（在"加载更多"按钮之后）
        insert_pos = 1 if self.load_more_button else 0

        # 在顶部插入新记录
        for record in new_records:
            count_before = self.messages_layout.count()
            self._render_record_at_position(record, insert_pos)
            widgets_added = self.messages_layout.count() - count_before
            insert_pos += widgets_added

        # 恢复滚动位置（使用两次延迟确保布局完全更新）
        def restore_scroll():
            # 强制处理布局更新
            self.messages_container.updateGeometry()
            # 再次延迟以确保滚动条范围已更新
            def do_restore():
                new_max = scrollbar.maximum()
                height_diff = new_max - old_max
                scrollbar.setValue(old_scroll_value + height_diff)
            QTimer.singleShot(50, do_restore)

        QTimer.singleShot(0, restore_scroll)

    def _render_record_at_position(self, record, position):
        """在指定位置渲染记录（用于增量加载）

        复用现有的 _render_record 方法，然后将新添加的 widget 移动到指定位置
        """
        # 记录当前 widget 数量
        count_before = self.messages_layout.count()

        # 使用现有方法添加记录（会添加到末尾）
        self._render_record(record)

        # 计算新添加的 widget 数量
        count_after = self.messages_layout.count()
        widgets_added = count_after - count_before

        # 从末尾取出新添加的 widgets，插入到指定位置
        for i in range(widgets_added):
            item = self.messages_layout.takeAt(count_before)
            if item and item.widget():
                self.messages_layout.insertWidget(position + i, item.widget())

    def _clear_messages(self):
        """清空所有消息"""
        while self.messages_layout.count():
            child = self.messages_layout.takeAt(0)
            if child.widget():
                child.widget().deleteLater()

    def _show_empty_message(self):
        """显示无历史记录提示"""
        empty_label = QLabel("暂无对话记录")
        empty_label.setObjectName("emptyStateLabel")
        empty_label.setAlignment(Qt.AlignCenter)
        self.messages_layout.addWidget(empty_label)

    def _setup_content_display(self, content: str) -> MarkdownDisplayWidget:
        """创建并配置内容显示组件（使用MarkdownDisplayWidget）"""
        content_display = MarkdownDisplayWidget()
        content_display.setMarkdownText(content)
        content_display.setStyleSheet('''
            QTextEdit {
                background-color: transparent;
                border: none;
                padding: 0px;
                color: #e0e0e0;
            }
        ''')
        content_display.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        content_display.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        content_display.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)

        # 根据内容自适应高度
        doc = content_display.document()
        doc.setTextWidth(content_display.viewport().width() if content_display.viewport().width() > 0 else 400)
        height = int(doc.size().height()) + 10
        content_display.setFixedHeight(height)

        return content_display

    def _create_avatar(self, text: str) -> QLabel:
        """创建头像标签"""
        label = QLabel(text)
        label.setObjectName("avatarLabel")
        label.setFixedSize(32, 32)
        label.setAlignment(Qt.AlignCenter)
        return label

    def _safe_set_text_later(self, widget, text: str, delay: int = 1000):
        """安全地延迟设置文本，使用弱引用避免访问已销毁对象"""
        weak_widget = weakref.ref(widget)

        def restore():
            w = weak_widget()
            if w is not None:
                try:
                    w.setText(text)
                except RuntimeError:
                    pass  # 对象已销毁

        QTimer.singleShot(delay, restore)

    def _copy_content(self, content: str, button: QPushButton):
        """复制内容到剪贴板"""
        try:
            pyperclip.copy(content)
            button.setText("✓")
            self._safe_set_text_later(button, "📋")
        except Exception as e:
            print(f"复制失败: {e}", file=sys.stderr)

    def _quote_content(self, msg_type: str, content: str, button: QPushButton):
        """生成引用格式并复制到剪贴板"""
        truncated = content[:100] + "..." if len(content) > 100 else content
        truncated = truncated.replace('\n', '\n> ')
        quote = f"----请回忆如下引用的历史对话内容----\n```quote\n[{msg_type}]\n{truncated}\n```\n---------"
        pyperclip.copy(quote)
        button.setText("✓")
        self._safe_set_text_later(button, "📎")

    def _save_content(self, content: str):
        """弹窗输入标题，保存为.md文件"""
        from PySide6.QtWidgets import QInputDialog
        title, ok = QInputDialog.getText(self, "保存文档", "请输入文档标题:")
        if ok and title:
            safe_title = "".join(c for c in title if c.isalnum() or c in (' ', '-', '_')).strip()
            if not safe_title:
                safe_title = "untitled"

            # 优先使用 workspace_path，否则回退到 project_path/.workspace/
            if self.workspace_path and os.path.exists(self.workspace_path):
                save_dir = self.workspace_path
            else:
                save_dir = os.path.join(self.project_path, '.workspace') if self.project_path else '.'

            os.makedirs(save_dir, exist_ok=True)
            file_path = os.path.join(save_dir, f"{safe_title}.md")

            with open(file_path, 'w', encoding='utf-8') as f:
                f.write(content)



    def _add_user_message(self, content: str):
        """添加用户消息（居左展示，与AI消息样式相同）"""
        row_widget = QWidget()
        row_layout = QHBoxLayout(row_widget)
        row_layout.setContentsMargins(0, 5, 0, 5)
        row_layout.setSpacing(10)

        # 1. 头像
        avatar = self._create_avatar("👤")
        row_layout.addWidget(avatar, alignment=Qt.AlignTop)

        # 2. 消息气泡容器
        bubble_container = QWidget()
        bubble_layout = QVBoxLayout(bubble_container)
        bubble_layout.setContentsMargins(0, 0, 0, 0)
        bubble_layout.setSpacing(2)

        # 角色标签和按钮
        header_widget = QWidget()
        header_layout = QHBoxLayout(header_widget)
        header_layout.setContentsMargins(0, 0, 0, 0)
        role_label = QLabel("User")
        role_label.setObjectName("roleLabel")
        header_layout.addWidget(role_label)

        copy_btn = QPushButton("📋")
        copy_btn.setFixedSize(24, 24)
        copy_btn.setStyleSheet("""
            QPushButton {
                background-color: transparent;
                border: none;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: rgba(255, 255, 255, 0.1);
                border-radius: 4px;
            }
        """)
        copy_btn.setCursor(Qt.PointingHandCursor)
        copy_btn.clicked.connect(lambda: self._copy_content(content, copy_btn))
        header_layout.addWidget(copy_btn)

        quote_btn = QPushButton("📎")
        quote_btn.setFixedSize(24, 24)
        quote_btn.setStyleSheet("""
            QPushButton {
                background-color: transparent;
                border: none;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: rgba(255, 255, 255, 0.1);
                border-radius: 4px;
            }
        """)
        quote_btn.setCursor(Qt.PointingHandCursor)
        quote_btn.clicked.connect(lambda: self._quote_content("用户消息", content, quote_btn))
        header_layout.addWidget(quote_btn)
        header_layout.addStretch()

        bubble_layout.addWidget(header_widget)

        # 气泡
        bubble = QFrame()
        bubble.setObjectName("aiBubble")  # 使用与AI相同的样式
        bubble_content_layout = QVBoxLayout(bubble)
        bubble_content_layout.setContentsMargins(12, 8, 12, 8)

        if content:
            content_display = self._setup_content_display(content)
            bubble_content_layout.addWidget(content_display)

        bubble_layout.addWidget(bubble)

        row_layout.addWidget(bubble_container, stretch=1)

        # 3. 右侧占位
        row_layout.addStretch(0)

        self.messages_layout.addWidget(row_widget)

    def _add_assistant_message(self, content: str):
        """添加AI消息"""
        row_widget = QWidget()
        row_layout = QHBoxLayout(row_widget)
        row_layout.setContentsMargins(0, 5, 0, 5)
        row_layout.setSpacing(10)

        # 1. 头像
        avatar = self._create_avatar("🤖")
        row_layout.addWidget(avatar, alignment=Qt.AlignTop)

        # 2. 消息气泡容器
        bubble_container = QWidget()
        bubble_layout = QVBoxLayout(bubble_container)
        bubble_layout.setContentsMargins(0, 0, 0, 0)
        bubble_layout.setSpacing(2)

        # 角色标签和复制按钮
        header_widget = QWidget()
        header_layout = QHBoxLayout(header_widget)
        header_layout.setContentsMargins(0, 0, 0, 0)
        role_label = QLabel("AI Assistant")
        role_label.setObjectName("roleLabel")
        header_layout.addWidget(role_label)

        copy_btn = QPushButton("📋")
        copy_btn.setFixedSize(24, 24)
        copy_btn.setStyleSheet("""
            QPushButton {
                background-color: transparent;
                border: none;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: rgba(255, 255, 255, 0.1);
                border-radius: 4px;
            }
        """)
        copy_btn.setCursor(Qt.PointingHandCursor)
        copy_btn.clicked.connect(lambda: self._copy_content(content, copy_btn))
        header_layout.addWidget(copy_btn)

        quote_btn = QPushButton("📎")
        quote_btn.setFixedSize(24, 24)
        quote_btn.setStyleSheet("""
            QPushButton {
                background-color: transparent;
                border: none;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: rgba(255, 255, 255, 0.1);
                border-radius: 4px;
            }
        """)
        quote_btn.setCursor(Qt.PointingHandCursor)
        quote_btn.clicked.connect(lambda: self._quote_content("AI回复", content, quote_btn))
        header_layout.addWidget(quote_btn)

        save_btn = QPushButton("💾")
        save_btn.setFixedSize(24, 24)
        save_btn.setStyleSheet("""
            QPushButton {
                background-color: transparent;
                border: none;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: rgba(255, 255, 255, 0.1);
                border-radius: 4px;
            }
        """)
        save_btn.setCursor(Qt.PointingHandCursor)
        save_btn.clicked.connect(lambda: self._save_content(content))
        header_layout.addWidget(save_btn)
        header_layout.addStretch()

        bubble_layout.addWidget(header_widget)

        # 气泡
        bubble = QFrame()
        bubble.setObjectName("aiBubble")
        bubble_content_layout = QVBoxLayout(bubble)
        bubble_content_layout.setContentsMargins(12, 8, 12, 8)

        if content:
            content_display = self._setup_content_display(content)
            bubble_content_layout.addWidget(content_display)

        bubble_layout.addWidget(bubble)

        row_layout.addWidget(bubble_container, stretch=1)

        # 3. 右侧占位
        row_layout.addStretch(0)

        self.messages_layout.addWidget(row_widget)

    def _add_feedback_messages(self, record: Dict):
        """将 feedback 工具拆分为两条消息：AI反馈 + 用户回复"""
        input_data = record.get('input', {})
        output = record.get('output', '')

        # 消息1: AI 反馈 (使用 assistant 样式)
        work_title = input_data.get('work_title', '')
        message = input_data.get('message', '')
        options = input_data.get('predefined_options', [])
        files = input_data.get('files', [])

        parts = []
        if work_title:
            parts.append(f"📢 **{work_title}**")
        if message:
            parts.append(message)
        if options:
            parts.append(f"**选项**: {' | '.join(options)}")
        if files:
            file_list = ', '.join([f"`{f}`" for f in files])
            parts.append(f"**相关文件**: {file_list}")

        ai_content = '\n\n'.join(parts) if parts else ''
        if ai_content:
            self._add_assistant_message(ai_content)

        # 消息2: 用户回复 (使用 user 样式)
        user_content = self._extract_user_feedback(output)
        if user_content:
            self._add_user_message(user_content)

    def _extract_user_feedback(self, output: str) -> str:
        """从 feedback output 中提取用户输入"""
        if not output:
            return ''
        # 提取 💬 用户输入 或 🔘 用户选择 后面的内容
        for marker in ['💬 用户输入：\n', '🔘 用户选择的选项：\n']:
            if marker in output:
                idx = output.find(marker)
                content = output[idx + len(marker):]
                # 截断到 💡 请注意 之前
                if '💡 请注意' in content:
                    end_idx = content.find('💡 请注意')
                    content = content[:end_idx].strip()
                return content
        return ''

    def _format_tool_input(self, name: str, input_data: Dict) -> str:
        """格式化工具输入为 markdown"""
        if name == 'Task':
            desc = input_data.get('description', '')
            prompt = input_data.get('prompt', '')
            agent_type = input_data.get('subagent_type', '')
            parts = []
            # 格式: Agent(agent名称):描述
            if agent_type and desc:
                parts.append(f"**Agent**({agent_type}):{desc}")
            elif agent_type:
                parts.append(f"**Agent**({agent_type})")
            elif desc:
                parts.append(f"**描述**: {desc}")
            if prompt:
                parts.append(f"**Prompt**:\n{prompt}")
            return '\n\n'.join(parts) if parts else str(input_data)
        elif name in ('Read', 'Glob', 'Grep'):
            file_path = input_data.get('file_path', input_data.get('path', ''))
            pattern = input_data.get('pattern', '')
            parts = []
            if file_path:
                parts.append(f"**路径**: `{file_path}`")
            if pattern:
                parts.append(f"**模式**: `{pattern}`")
            return '\n'.join(parts) if parts else str(input_data)
        elif name in ('Edit', 'Write'):
            file_path = input_data.get('file_path', '')
            return f"**文件**: `{file_path}`" if file_path else str(input_data)
        elif name == 'Hook':
            cmd = input_data.get('command', '')
            return f"**命令**: `{cmd}`" if cmd else str(input_data)
        elif 'feedback' in name.lower():
            # feedback 工具特殊处理
            work_title = input_data.get('work_title', '')
            message = input_data.get('message', '')
            options = input_data.get('predefined_options', [])
            files = input_data.get('files', [])
            parts = []
            if work_title:
                parts.append(f"📢 **{work_title}**")
            if message:
                parts.append(message)
            if options:
                parts.append(f"**选项**: {' | '.join(options)}")
            if files:
                parts.append(f"**相关文件**: {', '.join(files)}")
            return '\n\n'.join(parts) if parts else str(input_data)
        else:
            # 其他工具显示简化的 JSON
            input_str = json.dumps(input_data, ensure_ascii=False, indent=2)
            if len(input_str) > 300:
                input_str = input_str[:300] + "..."
            return f"```json\n{input_str}\n```"

    def _format_feedback_output(self, output: str) -> str:
        """格式化 feedback 工具的输出，提取用户输入"""
        if not output:
            return ''
        # 提取 💬 用户输入 或 🔘 用户选择 后面的内容
        for marker in ['💬 用户输入：\n', '🔘 用户选择的选项：\n']:
            if marker in output:
                idx = output.find(marker)
                content = output[idx + len(marker):]
                # 截断到 💡 请注意 之前
                if '💡 请注意' in content:
                    end_idx = content.find('💡 请注意')
                    content = content[:end_idx].strip()
                return f"**用户反馈**: {content}" if content else ''
        return ''

    def _add_tool_message(self, name: str, input_data: Dict, output: str, timestamp: str):
        """添加工具调用消息（默认折叠，feedback 默认展开）"""
        is_feedback = 'feedback' in name.lower()

        row_widget = QWidget()
        row_layout = QHBoxLayout(row_widget)
        row_layout.setContentsMargins(0, 2, 0, 2)
        row_layout.setSpacing(10)

        # 1. 头像
        avatar = self._create_avatar("💬" if is_feedback else "⚙️")
        row_layout.addWidget(avatar, alignment=Qt.AlignTop)

        # 2. 消息气泡容器
        bubble_container = QWidget()
        bubble_layout = QVBoxLayout(bubble_container)
        bubble_layout.setContentsMargins(0, 0, 0, 0)
        bubble_layout.setSpacing(2)

        # 生成标题文本
        # Task 工具特殊处理：显示 Agent(agent_type): description
        if name == 'Task':
            agent_type = input_data.get('subagent_type', '')
            desc = input_data.get('description', '')
            if agent_type and desc:
                header_title = f"Agent({agent_type}): {desc}"
            elif agent_type:
                header_title = f"Agent({agent_type})"
            else:
                header_title = f"Tool: {name}"
        else:
            header_title = f"Tool: {name}"

        # 可点击的标题（用于展开/折叠）+ 复制按钮
        # feedback 默认展开，其他默认折叠
        initial_expanded = is_feedback

        # Header 容器（包含折叠按钮和复制按钮）
        header_widget = QWidget()
        header_layout = QHBoxLayout(header_widget)
        header_layout.setContentsMargins(0, 0, 0, 0)
        header_layout.setSpacing(4)

        header_btn = QPushButton(f"{'▼' if initial_expanded else '▶'} {header_title}")
        header_btn.setObjectName("toolHeaderButton")
        # Task/Agent 使用绿色，其他工具使用灰色
        header_color = "#4CAF50" if name == 'Task' else "#888"
        header_hover_color = "#66BB6A" if name == 'Task' else "#aaa"
        header_btn.setStyleSheet(f"""
            QPushButton {{
                background-color: transparent;
                color: {header_color};
                border: none;
                text-align: left;
                padding: 2px 0;
                font-size: 12px;
            }}
            QPushButton:hover {{
                color: {header_hover_color};
                cursor: pointer;
            }}
        """)
        header_btn.setCursor(Qt.PointingHandCursor)
        header_layout.addWidget(header_btn)

        # 复制按钮
        copy_btn = QPushButton("📋")
        copy_btn.setFixedSize(24, 24)
        copy_btn.setStyleSheet("""
            QPushButton {
                background-color: transparent;
                border: none;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: rgba(255, 255, 255, 0.1);
                border-radius: 4px;
            }
        """)
        copy_btn.setCursor(Qt.PointingHandCursor)
        header_layout.addWidget(copy_btn)

        # 引用按钮
        quote_btn = QPushButton("📎")
        quote_btn.setFixedSize(24, 24)
        quote_btn.setStyleSheet("""
            QPushButton {
                background-color: transparent;
                border: none;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: rgba(255, 255, 255, 0.1);
                border-radius: 4px;
            }
        """)
        quote_btn.setCursor(Qt.PointingHandCursor)
        header_layout.addWidget(quote_btn)

        # 保存按钮
        save_btn = QPushButton("💾")
        save_btn.setFixedSize(24, 24)
        save_btn.setStyleSheet("""
            QPushButton {
                background-color: transparent;
                border: none;
                font-size: 14px;
            }
            QPushButton:hover {
                background-color: rgba(255, 255, 255, 0.1);
                border-radius: 4px;
            }
        """)
        save_btn.setCursor(Qt.PointingHandCursor)
        header_layout.addWidget(save_btn)
        header_layout.addStretch()

        bubble_layout.addWidget(header_widget)

        # 气泡（feedback 默认显示，其他默认隐藏）
        bubble = QFrame()
        bubble.setObjectName("aiBubble")
        bubble.setVisible(initial_expanded)
        bubble_content_layout = QVBoxLayout(bubble)
        bubble_content_layout.setContentsMargins(12, 8, 12, 8)

        # 格式化输入
        input_str = self._format_tool_input(name, input_data)

        # 输出内容
        output_str = str(output) if output else ''

        # 过滤掉 agentId 行（Agent 调用结果中的元信息）
        if output_str:
            lines = output_str.split('\n')
            filtered_lines = [line for line in lines if not line.strip().startswith('agentId:')]
            output_str = '\n'.join(filtered_lines).strip()

        # 处理 base64 图片 (只有真正的 base64 数据才替换)
        # 真正的 base64 图片数据通常包含 data:image 或者是纯 base64 编码的长字符串
        if output_str and len(output_str) > 500:
            # 检测是否为 base64 图片数据
            is_base64_image = (
                'data:image' in output_str.lower() or
                (output_str.startswith('/9j/') or output_str.startswith('iVBOR'))  # JPEG/PNG base64 头
            )
            if is_base64_image:
                output_str = "[图片]"

        # feedback 工具特殊处理输出
        if is_feedback:
            output_str = self._format_feedback_output(output_str)

        # 构建内容（不再截断，展示完整内容）
        content_parts = [f"**Input:**\n{input_str}"]
        if output_str:
            content_parts.append(f"**Output:**\n{output_str}")
        else:
            content_parts.append("**Output:** (无输出)")
        content = '\n\n'.join(content_parts)
        content_display = self._setup_content_display(content)
        bubble_content_layout.addWidget(content_display)

        bubble_layout.addWidget(bubble)

        # 复制按钮点击事件（复制完整内容：Input + Output）
        copy_btn.clicked.connect(lambda: self._copy_content(content, copy_btn))

        # 引用按钮点击事件
        quote_btn.clicked.connect(lambda: self._quote_content(f"工具调用: {name}", content, quote_btn))

        # 保存按钮点击事件
        save_btn.clicked.connect(lambda: self._save_content(content))

        # 点击展开/折叠
        def toggle_content():
            is_visible = bubble.isVisible()
            bubble.setVisible(not is_visible)
            header_btn.setText(f"{'▼' if not is_visible else '▶'} {header_title}")

        header_btn.clicked.connect(toggle_content)

        row_layout.addWidget(bubble_container, stretch=1)

        # 3. 右侧占位
        row_layout.addStretch(0)

        self.messages_layout.addWidget(row_widget)

    def _load_history_from_file(self) -> List[Dict]:
        """从Claude Code的session .jsonl文件加载历史记录"""
        try:
            if not self.session_id or not self.project_path:
                return []

            # 编码项目路径 (Claude Code 将 / 和 _ 都替换为 -)
            encoded_path = self.project_path.replace('/', '-').replace('_', '-')

            # 构建 .jsonl 文件路径
            home_dir = os.path.expanduser('~')
            jsonl_file = os.path.join(home_dir, '.claude', 'projects', encoded_path, f'{self.session_id}.jsonl')

            if not os.path.exists(jsonl_file):
                return []

            # 读取所有行
            lines = []
            with open(jsonl_file, 'r', encoding='utf-8') as f:
                lines = f.readlines()

            # 第一遍：收集所有 tool_results
            tool_results = {}
            for line in lines:
                line = line.strip()
                if not line:
                    continue
                try:
                    entry = json.loads(line)
                    message = entry.get('message', {})
                    if message.get('role') != 'user':
                        continue
                    content = message.get('content', [])
                    if isinstance(content, list):
                        for item in content:
                            if isinstance(item, dict) and item.get('type') == 'tool_result':
                                tool_use_id = item.get('tool_use_id')
                                tool_content = item.get('content', '')
                                # 处理 content 为数组的情况
                                if isinstance(tool_content, list):
                                    texts = []
                                    for c in tool_content:
                                        if isinstance(c, dict) and c.get('type') == 'text':
                                            texts.append(c.get('text', ''))
                                    tool_content = '\n'.join(texts)
                                if tool_use_id:
                                    tool_results[tool_use_id] = tool_content
                except json.JSONDecodeError:
                    continue

            # 第二遍：构建消息列表
            messages = []
            for line in lines:
                line = line.strip()
                if not line:
                    continue

                try:
                    entry = json.loads(line)
                    message = entry.get('message', {})
                    role = message.get('role')

                    # 处理 system 消息 (hook)
                    entry_type = entry.get('type')
                    if entry_type == 'system':
                        subtype = entry.get('subtype', '')
                        if subtype == 'stop_hook_summary':
                            hook_infos = entry.get('hookInfos', [])
                            hook_errors = entry.get('hookErrors', [])
                            hook_cmd = hook_infos[0].get('command', '') if hook_infos else ''
                            # hookErrors 实际上是 hook 的输出内容
                            hook_output = '\n'.join(hook_errors) if hook_errors else '执行完成'
                            messages.append({
                                'role': 'tool',
                                'name': 'Hook',
                                'input': {'command': hook_cmd},
                                'output': hook_output,
                                'timestamp': entry.get('timestamp', '')
                            })
                        continue

                    if role not in ['user', 'assistant']:
                        continue

                    timestamp = entry.get('timestamp', '')
                    content = message.get('content', [])

                    # 处理 user 消息
                    if role == 'user':
                        if isinstance(content, str):
                            # 过滤 hook 注入的内容（在 "Stop hook feedback:" 或 "hook feedback:" 后面）
                            user_content = content
                            for marker in ['Stop hook feedback:\n', 'hook feedback:\n']:
                                if marker in user_content:
                                    # 只保留 marker 之前的内容 + marker 本身
                                    idx = user_content.find(marker)
                                    user_content = user_content[:idx + len(marker)].rstrip()
                                    break
                            if user_content:
                                messages.append({'role': 'user', 'content': user_content, 'timestamp': timestamp})
                        # tool_result 不作为独立消息显示

                    # 处理 assistant 消息
                    elif role == 'assistant':
                        if isinstance(content, list):
                            for item in content:
                                if not isinstance(item, dict):
                                    continue

                                item_type = item.get('type')

                                # 文本消息
                                if item_type == 'text':
                                    text = item.get('text', '')
                                    if text:
                                        messages.append({'role': 'assistant', 'content': text, 'timestamp': timestamp})

                                # 工具调用
                                elif item_type == 'tool_use':
                                    tool_id = item.get('id')
                                    tool_name = item.get('name', '')
                                    tool_input = item.get('input', {})
                                    tool_output = tool_results.get(tool_id, '')

                                    messages.append({
                                        'role': 'tool',
                                        'name': tool_name,
                                        'input': tool_input,
                                        'output': tool_output,
                                        'timestamp': timestamp
                                    })

                except json.JSONDecodeError:
                    continue

            return messages

        except Exception as e:
            print(f"加载历史记录失败: {e}", file=sys.stderr)
            return []


    def refresh_history(self):
        """刷新历史记录"""
        self.load_history()
        self._scroll_to_bottom()

    def _scroll_to_bottom(self):
        """滚动到底部"""
        weak_scroll = weakref.ref(self.scroll_area)

        def do_scroll():
            scroll = weak_scroll()
            if scroll is not None:
                try:
                    scroll.verticalScrollBar().setValue(scroll.verticalScrollBar().maximum())
                except RuntimeError:
                    pass  # 对象已销毁

        QTimer.singleShot(100, do_scroll)

    def showEvent(self, event):
        """Tab显示时加载历史记录并滚动到底部"""
        super().showEvent(event)
        if not self._loaded:
            self._loaded = True
            self.load_history()
        self._scroll_to_bottom()



"""
指令管理模块
包含指令的增删改查、UI组件等所有相关功能
"""

import os
import sys
import json
from typing import Optional, List, Dict, Any

try:
    from PySide6.QtWidgets import (
        QWidget, QVBoxLayout, QHBoxLayout, QTabWidget, QScrollArea,
        QGridLayout, QRadioButton, QButtonGroup, QPushButton, QFrame,
        QLabel, QSizePolicy, QDialog
    )
    from PySide6.QtCore import Qt, Signal
except ImportError:
    from PyQt5.QtWidgets import (
        QWidget, QVBoxLayout, QHBoxLayout, QTabWidget, QScrollArea,
        QGridLayout, QRadioButton, QButtonGroup, QPushButton, QFrame,
        QLabel, QSizePolicy, QDialog
    )
    from PyQt5.QtCore import Qt, pyqtSignal as Signal

# 导入指令对话框
try:
    import add_command_dialog
    AddCommandDialog = add_command_dialog.AddCommandDialog
    EditCommandDialog = add_command_dialog.EditCommandDialog
except ImportError:
    try:
        from add_command_dialog import AddCommandDialog, EditCommandDialog
    except ImportError:
        print("Warning: 无法导入命令对话框组件")
        AddCommandDialog = None
        EditCommandDialog = None

# 导入路径配置模块
try:
    from path_config import get_path_config
    PATH_CONFIG_AVAILABLE = True
except ImportError:
    PATH_CONFIG_AVAILABLE = False

# 导入调试日志模块
try:
    from debug_logger import get_debug_logger
    DEBUG_LOG_AVAILABLE = True
except ImportError:
    DEBUG_LOG_AVAILABLE = False


class CommandManager:
    """指令数据管理类"""

    def __init__(self, project_path: str):
        self.project_path = project_path
        # 获取路径配置实例
        if PATH_CONFIG_AVAILABLE:
            self.path_config = get_path_config()
        else:
            self.path_config = None
        # 命令缓存
        self._cache = {}
        self._cache_enabled = False
    
    def enable_cache(self):
        """启用缓存模式"""
        self._cache_enabled = True
        self._cache.clear()

    def disable_cache(self):
        """禁用缓存模式"""
        self._cache_enabled = False
        self._cache.clear()

    def load_project_commands(self) -> List[Dict[str, Any]]:
        """加载项目指令（.cursor/rules/目录）"""
        if self._cache_enabled and 'project' in self._cache:
            return self._cache['project']

        commands = []
        if not self.project_path:
            return commands

        if self.path_config:
            prompts_dir = self.path_config.get_project_commands_dir(self.project_path)
            if prompts_dir:
                commands = self._load_commands_from_dir(prompts_dir, "项目")

        if self._cache_enabled:
            self._cache['project'] = commands
        return commands
    
    def load_personal_commands(self) -> List[Dict[str, Any]]:
        """个人指令已移除，返回空列表"""
        return []

    def _convert_to_marketplace_path(self, cache_path: str) -> str:
        """将 cache 路径转换为 marketplaces 路径（优先使用最新版本）"""
        # 匹配: ~/.claude/plugins/cache/{marketplace}/{plugin}/{version}
        # 转换: ~/.claude/plugins/marketplaces/{marketplace}/plugins/{plugin}

        cache_marker = '/plugins/cache/'
        if cache_marker not in cache_path:
            return cache_path  # 非 cache 路径，原样返回

        try:
            # 提取 marketplace 和 plugin 名称
            parts = cache_path.split(cache_marker)[1].split('/')
            if len(parts) >= 2:
                marketplace = parts[0]  # cc-marketplace
                plugin_name = parts[1]  # gemini

                # 构建 marketplaces 路径
                claude_dir = os.path.expanduser("~/.claude")
                marketplace_path = os.path.join(
                    claude_dir, "plugins", "marketplaces",
                    marketplace, "plugins", plugin_name
                )

                if os.path.exists(marketplace_path):
                    return marketplace_path
        except Exception:
            pass

        return cache_path  # 转换失败，返回原路径

    def load_plugin_commands(self) -> List[Dict[str, Any]]:
        """加载插件指令（从已启用插件的commands目录）"""
        if self._cache_enabled and 'plugin' in self._cache:
            return self._cache['plugin']

        commands = []

        if not self.path_config:
            return commands

        try:
            # 1. 读取 installed_plugins.json
            plugins_config_path = self.path_config.get_plugins_config_path()
            if not os.path.exists(plugins_config_path):
                if DEBUG_LOG_AVAILABLE:
                    logger = get_debug_logger()
                    logger.log("插件配置文件不存在", "WARN")
                return commands

            with open(plugins_config_path, 'r', encoding='utf-8') as f:
                plugins_data = json.load(f)

            installed_plugins = plugins_data.get('plugins', {})

            # 2. 读取 settings.json 获取启用状态
            settings_path = self.path_config.get_settings_path()
            enabled_plugins = {}
            if os.path.exists(settings_path):
                with open(settings_path, 'r', encoding='utf-8') as f:
                    settings_data = json.load(f)
                enabled_plugins = settings_data.get('enabledPlugins', {})

            # 3. 遍历已启用的插件
            for plugin_id, plugin_info in installed_plugins.items():
                # 只加载已启用的插件
                if not enabled_plugins.get(plugin_id, False):
                    continue

                # plugin_info 可能是数组（新格式）或对象（旧格式）
                if isinstance(plugin_info, list) and len(plugin_info) > 0:
                    install_path = plugin_info[0].get('installPath')
                else:
                    install_path = plugin_info.get('installPath')

                # 尝试转换为 marketplaces 路径（优先使用最新版本）
                if install_path:
                    install_path = self._convert_to_marketplace_path(install_path)

                if not install_path or not os.path.exists(install_path):
                    if DEBUG_LOG_AVAILABLE:
                        logger = get_debug_logger()
                        logger.log(f"插件路径不存在: {plugin_id}", "WARN")
                    continue

                # 4. 扫描插件的 commands/ 目录
                commands_dir = os.path.join(install_path, 'commands')
                if not os.path.exists(commands_dir):
                    continue

                # 5. 复用 _load_commands_from_dir 方法
                plugin_commands = self._load_commands_from_dir(commands_dir, "插件")

                # 6. 为每个指令添加插件标识
                plugin_name = plugin_id.split('@')[0] if '@' in plugin_id else plugin_id
                for cmd in plugin_commands:
                    cmd['plugin_id'] = plugin_id
                    cmd['plugin_name'] = plugin_name
                    # 修改 title 格式：plugin_name:title
                    original_title = cmd['title']
                    cmd['title'] = f"{plugin_name}:{original_title}"

                commands.extend(plugin_commands)

        except json.JSONDecodeError as e:
            sys.stderr.write(f"Error parsing plugin config JSON: {e}\n")
        except Exception as e:
            sys.stderr.write(f"Error loading plugin commands: {e}\n")

        if self._cache_enabled:
            self._cache['plugin'] = commands
        return commands

    def _load_commands_from_dir(self, prompts_dir: str, path_type: str) -> List[Dict[str, Any]]:
        """从指定目录加载指令（支持递归读取子目录）"""
        commands = []

        if DEBUG_LOG_AVAILABLE:
            logger = get_debug_logger()
            logger.log(f"_load_commands_from_dir: 目录={prompts_dir}, 类型={path_type}", "LOAD")

        if not os.path.exists(prompts_dir):
            if DEBUG_LOG_AVAILABLE:
                logger = get_debug_logger()
                logger.log_error("目录扫描", f"目录不存在: {prompts_dir}")
            return commands

        # 递归函数来读取目录和子目录
        def scan_directory(dir_path: str, relative_path: str = "") -> None:
            try:
                entries = os.listdir(dir_path)

                if DEBUG_LOG_AVAILABLE and dir_path == prompts_dir:
                    logger = get_debug_logger()
                    md_files = [f for f in entries if f.endswith('.md')]
                    logger.log_dir_scan(dir_path, entries, md_files)

                for entry in entries:
                    entry_path = os.path.join(dir_path, entry)

                    if os.path.isdir(entry_path):
                        # 递归读取子目录
                        new_relative = os.path.join(relative_path, entry) if relative_path else entry
                        scan_directory(entry_path, new_relative)
                    elif entry.endswith('.md'):
                        try:
                            with open(entry_path, 'r', encoding='utf-8') as f:
                                content = f.read().strip()

                            if not content:
                                continue

                            # 生成标题：使用冒号分隔的格式
                            file_title = entry.replace('.md', '')
                            if relative_path:
                                # 将路径分隔符替换为冒号
                                path_parts = relative_path.replace(os.sep, ':')
                                title = f"{path_parts}:{file_title}"
                            else:
                                title = file_title

                            description = ""
                            main_content = content
                            globs = ""
                            always_apply = False
                            argument_hint = ""  # 新增：参数提示字段

                            if content.startswith('---'):
                                # 分离frontmatter和内容
                                parts = content.split('---', 2)
                                if len(parts) >= 3:
                                    frontmatter = parts[1].strip()
                                    main_content = parts[2].strip()

                                    # 解析frontmatter中的各个字段
                                    for line in frontmatter.split('\n'):
                                        line = line.strip()
                                        if line.startswith('title:'):
                                            extracted_title = line.split('title:', 1)[1].strip()
                                            if extracted_title:  # 如果frontmatter中有title，使用它但保留路径前缀
                                                if relative_path:
                                                    path_parts = relative_path.replace(os.sep, ':')
                                                    title = f"{path_parts}:{extracted_title}"
                                                else:
                                                    title = extracted_title
                                        elif line.startswith('description:'):
                                            description = line.split('description:', 1)[1].strip()
                                        elif line.startswith('argument-hint:'):
                                            argument_hint = line.split('argument-hint:', 1)[1].strip()
                                        elif line.startswith('globs:'):
                                            globs = line.split('globs:', 1)[1].strip()
                                        elif line.startswith('alwaysApply:'):
                                            always_apply_str = line.split('alwaysApply:', 1)[1].strip().lower()
                                            always_apply = always_apply_str in ('true', 'yes', '1')
                            else:
                                # 没有frontmatter的文件，使用文件名作为描述，内容作为主体
                                description = f"来自文件: {os.path.join(relative_path, entry) if relative_path else entry}"

                            # 只有当有实际内容时才添加指令
                            if main_content:
                                commands.append({
                                    'title': title,
                                    'content': main_content,
                                    'description': description,
                                    'filename': entry,
                                    'path_type': path_type,
                                    'full_path': entry_path,
                                    'globs': globs,
                                    'always_apply': always_apply,
                                    'argument_hint': argument_hint,  # 新增：参数提示
                                    'relative_path': relative_path  # 添加相对路径信息
                                })
                        except Exception as e:
                            # Error loading file - log to stderr instead of stdout
                            sys.stderr.write(f"Error loading {entry} from {path_type}: {e}\n")
            except Exception as e:
                sys.stderr.write(f"Error scanning directory {dir_path}: {e}\n")

        # 开始递归扫描
        try:
            scan_directory(prompts_dir)
        except Exception as e:
            # Error reading directory - log to stderr instead of stdout  
            sys.stderr.write(f"Error reading {path_type} prompts directory: {e}\n")
        
        return commands


class CommandTabWidget(QTabWidget):
    """指令选项卡组件"""
    command_executed = Signal(str)  # 指令执行信号
    
    def __init__(self, project_path: str, parent=None):
        super().__init__(parent)
        self.project_path = project_path
        self.command_manager = CommandManager(project_path)
        
        # 设置大小策略：优先按内容大小，但允许收缩
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum)
        # 限制TabWidget的最大高度为200px，但允许按内容自适应
        self.setMaximumHeight(200)
        
        self._create_tabs()
        self._setup_tab_change_handler()
        # 设置初始按钮显示
        self._on_tab_changed(0)
        # 确保按钮容器正确初始化
        self._ensure_button_container_initialized()
    
    def _create_tabs(self):
        """创建项目指令和个人指令选项卡"""
        # 项目指令选项卡
        self.project_tab = ProjectCommandTab(self.project_path, self.command_manager, self)
        self.project_tab.command_executed.connect(self.command_executed.emit)
        self.project_tab.commands_changed.connect(self._on_commands_changed)
        self.addTab(self.project_tab, "🏢 项目指令")

        # 个人指令选项卡
        self.personal_tab = PersonalCommandTab(self.project_path, self.command_manager, self)
        self.personal_tab.command_executed.connect(self.command_executed.emit)
        self.personal_tab.commands_changed.connect(self._on_commands_changed)
        self.addTab(self.personal_tab, "👤 个人指令")

    
    def _setup_tab_change_handler(self):
        """设置选项卡切换处理"""
        self.currentChanged.connect(self._on_tab_changed)
    
    def _on_tab_changed(self, index):
        """选项卡切换时更新按钮"""
        print(f"🔄 选项卡切换到索引: {index}")
        
        # 获取当前选项卡
        current_tab = self.widget(index)
        if current_tab and hasattr(current_tab, 'get_button_container'):
            button_container = current_tab.get_button_container()
            if button_container:
                # 确保按钮容器可见
                button_container.setVisible(True)
                self.setCornerWidget(button_container, Qt.TopRightCorner)
                print(f"✅ 设置按钮容器成功，选项卡: {self.tabText(index)}")
            else:
                print(f"⚠️  当前选项卡 '{self.tabText(index)}' 没有按钮容器，保持现有按钮")
                # 如果当前选项卡没有按钮容器，不要清空现有的cornerWidget
                # 这样可以保持之前有按钮的选项卡的按钮仍然可见
        else:
            print(f"❌ 选项卡 '{self.tabText(index)}' 没有get_button_container方法")
    
    def _on_commands_changed(self):
        """指令变化时的处理"""
        # 刷新所有选项卡的数据
        self.project_tab.refresh_commands()
        self.personal_tab.refresh_commands()

    def refresh_all_commands(self):
        """刷新所有指令"""
        self.project_tab.refresh_commands()
        self.personal_tab.refresh_commands()
    
    def _ensure_button_container_initialized(self):
        """确保按钮容器正确初始化"""
        # 强制设置第一个有按钮容器的选项卡的按钮
        for i in range(self.count()):
            tab = self.widget(i)
            if tab and hasattr(tab, 'get_button_container'):
                button_container = tab.get_button_container()
                if button_container:
                    button_container.setVisible(True)
                    self.setCornerWidget(button_container, Qt.TopRightCorner)
                    print(f"✅ 初始化按钮容器成功，选项卡索引: {i}")
                    break


class BaseCommandTab(QWidget):
    """指令选项卡基类"""
    command_executed = Signal(str)  # 指令执行信号
    commands_changed = Signal()     # 指令变化信号
    
    def __init__(self, project_path: str, command_manager: CommandManager, command_type: str, parent=None):
        super().__init__(parent)
        self.project_path = project_path
        self.command_manager = command_manager
        self.command_type = command_type  # "项目" 或 "个人"
        self.commands = []
        self.command_button_group = QButtonGroup()
        self.command_button_group.setExclusive(False)  # Allow deselection
        self.command_radios = []
        self.button_container = None
        
        self._create_ui()
        self._create_button_container()
        self.refresh_commands()
    
    def _create_ui(self):
        """创建UI"""
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(5)
        
        # 创建主内容区域
        self.content_widget = QWidget()
        self.content_layout = QVBoxLayout(self.content_widget)
        self.content_layout.setContentsMargins(0, 0, 0, 0)
        self.content_layout.setSpacing(5)
        
        layout.addWidget(self.content_widget)
    
    def _create_button_container(self):
        """创建按钮容器 - 已禁用所有管理按钮"""
        # 不再创建添加、编辑按钮,用户需直接编辑 .md 文件来管理指令
        self.button_container = None
    
    def get_button_container(self):
        """获取按钮容器 - 已禁用"""
        return None  # 不再提供按钮容器
    
    def _load_commands(self) -> List[Dict[str, Any]]:
        """加载指令（子类实现）"""
        raise NotImplementedError
    
    def _get_default_command_type_for_dialog(self) -> str:
        """获取对话框的默认指令类型（子类实现）"""
        raise NotImplementedError
    
    def refresh_commands(self):
        """刷新指令列表"""
        # 清除旧的UI组件
        for i in reversed(range(self.content_layout.count())):
            child = self.content_layout.itemAt(i).widget()
            if child:
                child.setParent(None)
        
        # 清除按钮组
        self.command_radios.clear()
        
        # 重新加载指令
        self.commands = self._load_commands()

        # 编辑按钮已移除,不再更新状态
        
        if self.commands:
            self._create_command_list()
        else:
            self._create_empty_state()
    
    def _create_command_list(self):
        """创建指令列表"""
        # 创建滚动区域
        scroll_area = QScrollArea()
        scroll_area.setWidgetResizable(True)
        scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        scroll_area.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        
        commands_widget = QWidget()
        commands_layout = QGridLayout(commands_widget)
        commands_layout.setContentsMargins(5, 5, 5, 5)
        commands_layout.setSpacing(5)
        
        # 计算网格布局
        total_commands = len(self.commands)
        columns = 2  # 两列布局
        
        for i, command in enumerate(self.commands):
            row = i // columns
            col = i % columns
            
            # Create frame for each command item
            command_frame = QFrame()
            command_item_layout = QHBoxLayout(command_frame)
            command_item_layout.setContentsMargins(5, 2, 5, 2)

            # Radio button (只显示圆点)
            radio = QRadioButton()
            radio.setProperty('command_index', i)
            radio.clicked.connect(lambda checked, r=radio: self._handle_radio_click(r))
            # 设置选中效果样式
            radio.setStyleSheet("""
                QRadioButton {
                    padding: 4px 0px;
                }
                QRadioButton::indicator {
                    width: 14px;
                    height: 14px;
                    border-radius: 7px;
                    border: 2px solid #666666;
                }
                QRadioButton::indicator:checked {
                    background-color: #0078d4;
                    border-color: #0078d4;
                }
                QRadioButton::indicator:hover {
                    border-color: #0078d4;
                }
            """)
            self.command_button_group.addButton(radio)
            self.command_radios.append(radio)

            # 标题标签（支持富文本，插件名灰色，指令名白色）
            title = command['title']
            if ':' in title:
                parts = title.split(':', 1)
                formatted_title = f'<span style="color: #888888;">{parts[0]}:</span><span style="color: #ffffff;">{parts[1]}</span>'
            else:
                formatted_title = title
            title_label = QLabel(formatted_title)
            title_label.setTextFormat(Qt.RichText)
            title_label.setStyleSheet("""
                QLabel {
                    padding: 4px 8px;
                    border-radius: 3px;
                }
                QLabel:hover {
                    background-color: rgba(0, 120, 212, 0.08);
                }
            """)
            # 点击标签时也触发 radio 选中
            title_label.mousePressEvent = lambda event, r=radio: (r.setChecked(True), self._handle_radio_click(r))
            
            # Button container
            button_layout = QHBoxLayout()
            button_layout.setSpacing(10)
            
            # Execute button
            execute_btn = QPushButton("▶️")
            execute_btn.setMaximumSize(30, 30)
            execute_btn.setProperty('command_index', i)
            execute_btn.clicked.connect(lambda checked, idx=i: self._execute_command(idx))
            execute_btn.setToolTip("立即执行")
            execute_btn.setStyleSheet("""
                QPushButton {
                    background: transparent;
                    border: none;
                    font-size: 16px;
                }
                QPushButton:hover {
                    background-color: rgba(76, 175, 80, 0.1);
                    border-radius: 3px;
                }
                QPushButton:pressed {
                    background-color: rgba(76, 175, 80, 0.2);
                    border-radius: 3px;
                }
            """)
            button_layout.addWidget(execute_btn)
            
            # Add to frame layout
            command_item_layout.addWidget(radio)
            command_item_layout.addWidget(title_label)
            command_item_layout.addStretch()
            command_item_layout.addLayout(button_layout)
            
            # Add frame to grid layout
            commands_layout.addWidget(command_frame, row, col)
        
        commands_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
        scroll_area.setWidget(commands_widget)
        self.content_layout.addWidget(scroll_area)
    
    def _create_empty_state(self):
        """创建空状态提示"""
        empty_widget = QWidget()
        empty_layout = QVBoxLayout(empty_widget)
        empty_layout.setContentsMargins(20, 20, 20, 20)
        
        empty_layout.addStretch()
        
        # 主提示标签
        empty_label = QLabel(f"💡 暂无{self.command_type}指令")
        empty_label.setAlignment(Qt.AlignCenter)
        empty_label.setStyleSheet("""
            QLabel {
                font-size: 16px;
                font-weight: bold;
                color: #666666;
                margin-bottom: 10px;
            }
        """)
        empty_layout.addWidget(empty_label)
        
        # 操作提示标签
        help_label = QLabel("点击右上角 ➕ 按钮添加您的第一个指令")
        help_label.setAlignment(Qt.AlignCenter)
        help_label.setStyleSheet("""
            QLabel {
                font-size: 12px;
                color: #888888;
                margin-top: 5px;
            }
        """)
        empty_layout.addWidget(help_label)
        
        empty_layout.addStretch()
        
        self.content_layout.addWidget(empty_widget)
    
    def _handle_radio_click(self, radio_button):
        """处理radio按钮点击"""
        if radio_button.isChecked():
            # 取消其他radio的选中状态
            for other_radio in self.command_radios:
                if other_radio != radio_button and other_radio.isChecked():
                    other_radio.setChecked(False)

        # 编辑按钮已移除,不再更新状态
    
    def _execute_command(self, command_index: int):
        """执行指令"""
        if 0 <= command_index < len(self.commands):
            command_content = self.commands[command_index]['content']
            if command_content:
                self.command_executed.emit(command_content)
    
    # 添加/编辑指令方法已移除
    # 用户需要直接编辑 .md 文件来管理指令


class ProjectCommandTab(BaseCommandTab):
    """项目指令选项卡"""
    
    def __init__(self, project_path: str, command_manager: CommandManager, parent=None):
        super().__init__(project_path, command_manager, "项目", parent)
    
    def _load_commands(self) -> List[Dict[str, Any]]:
        """加载项目指令"""
        return self.command_manager.load_project_commands()
    
    def _get_default_command_type_for_dialog(self) -> str:
        """获取对话框的默认指令类型"""
        return "project"
    



class PersonalCommandTab(BaseCommandTab):
    """个人指令选项卡"""
    
    def __init__(self, project_path: str, command_manager: CommandManager, parent=None):
        super().__init__(project_path, command_manager, "个人", parent)
    
    def _load_commands(self) -> List[Dict[str, Any]]:
        """加载个人指令"""
        return self.command_manager.load_personal_commands()
    
    def _get_default_command_type_for_dialog(self) -> str:
        """获取对话框的默认指令类型"""
        return "private"
 


"""
UI 模块 - 导出UI组件
"""

# 紧凑反馈界面
try:
    from .compact_feedback_ui import CompactFeedbackUI
except ImportError:
    CompactFeedbackUI = None

# 会话列表界面
try:
    from .session_list_ui import SessionListUI
except ImportError:
    SessionListUI = None

__all__ = [
    'CompactFeedbackUI',
    'SessionListUI'
] 


#!/usr/bin/env python3
"""
Stop Hook处理脚本
智能处理stop事件，通过检查AI消息中的停止标记来决定是否允许停止
"""
import sys
import json
import os
from pathlib import Path

# 添加当前目录到Python路径
current_dir = Path(__file__).parent
sys.path.insert(0, str(current_dir))

from context_formatter import format_for_stop_hook

# 停止标记常量
STOP_MARKER = "<用户已明确告知停止工作>"


def check_stop_marker_in_transcript(transcript_path: str) -> bool:
    """检查 transcript 中最后一条 AI 消息是否包含停止标记

    Args:
        transcript_path: 对话记录文件路径 (JSONL格式)

    Returns:
        如果最后一条 AI 消息包含停止标记则返回 True
    """
    try:
        if not transcript_path:
            return False

        path = Path(transcript_path)
        if not path.exists():
            return False

        # 读取 JSONL 文件，找最后一条 assistant 消息
        last_assistant_msg = None
        with open(path, 'r', encoding='utf-8') as f:
            for line in f:
                line = line.strip()
                if not line:
                    continue
                try:
                    entry = json.loads(line)
                    if entry.get('type') == 'assistant':
                        last_assistant_msg = entry
                except json.JSONDecodeError:
                    continue

        if not last_assistant_msg:
            return False

        # 检查消息内容是否包含停止标记
        message = last_assistant_msg.get('message', {})
        content = message.get('content', [])

        for item in content:
            if isinstance(item, dict) and item.get('type') == 'text':
                text = item.get('text', '')
                if STOP_MARKER in text:
                    return True
            elif isinstance(item, str):
                if STOP_MARKER in item:
                    return True

        return False
    except Exception:
        return False


def main():
    """主函数"""
    try:
        # 从stdin读取JSON输入
        input_data = json.load(sys.stdin)

        # 提取关键信息
        session_id = input_data.get('session_id', '')
        transcript_path = input_data.get('transcript_path', '')

        # 获取项目路径
        project_path = os.getcwd()

        # 检查 transcript 中是否有停止标记
        if check_stop_marker_in_transcript(transcript_path):
            # AI 已声明停止，允许停止
            return 0

        # 默认行为：阻止停止并提示使用feedback工具
        if session_id:
            reason_text = format_for_stop_hook(session_id, project_path)
        else:
            reason_text = "请你调用 feedback mcp tool 向用户反馈/请示。示例：使用 mcp__feedback__feedback 工具向用户汇报当前工作进度、完成状态或请求下一步指示。"

        result = {
            "decision": "block",
            "reason": reason_text
        }
        print(json.dumps(result, ensure_ascii=False))
        return 0

    except Exception as e:
        # 发生错误时，默认允许停止（避免卡死）
        error_result = {
            "decision": "approve",
            "reason": f"Hook处理出错: {str(e)}"
        }
        print(json.dumps(error_result, ensure_ascii=False), file=sys.stderr)
        return 1

if __name__ == "__main__":
    sys.exit(main())



"""
Markdown显示组件

用于渲染和显示Markdown内容的只读文本框。
"""

import sys
import re
import markdown
import os

from PySide6.QtWidgets import QTextEdit, QDialog, QVBoxLayout, QLabel
from PySide6.QtCore import Qt, QUrl
from PySide6.QtGui import QPixmap, QMouseEvent, QTextDocument, QDesktopServices


class ImageViewerDialog(QDialog):
    """简单的图片查看对话框"""
    def __init__(self, image_path: str, parent=None):
        super().__init__(parent)
        self.setWindowTitle("图片查看")
        self.setModal(True)

        layout = QVBoxLayout(self)
        label = QLabel()
        pixmap = QPixmap(image_path)
        if not pixmap.isNull():
            # 限制最大尺寸为屏幕的80%
            screen = self.screen().geometry()
            max_width = int(screen.width() * 0.8)
            max_height = int(screen.height() * 0.8)
            if pixmap.width() > max_width or pixmap.height() > max_height:
                pixmap = pixmap.scaled(max_width, max_height, Qt.KeepAspectRatio, Qt.SmoothTransformation)
            label.setPixmap(pixmap)
        layout.addWidget(label)
        self.resize(pixmap.width(), pixmap.height())


class MarkdownDisplayWidget(QTextEdit):
    """支持Markdown格式显示的组件"""
    
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setReadOnly(True)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.setMouseTracking(True)  # 启用鼠标追踪以支持hover效果

        # 设置样式，去掉边框，更简洁的文档显示区域
        self.setStyleSheet("""
            QTextEdit {
                background-color: #2b2b2b;
                border: none;
                padding: 0px;
                color: white;
                font-family: 'Segoe UI', 'Arial', sans-serif;
                font-size: 13px;
                line-height: 1.4;
            }
        """)
    
    def setMarkdownText(self, markdown_text: str):
        """设置Markdown文本并渲染为HTML"""
        try:
            # 预处理：转换图片路径为HTML
            def _convert_image_paths(text: str) -> tuple[str, list[str]]:
                pattern = r'\[图片(\d+):\s*(.+?)\]'
                image_paths = []
                def replace_image(match):
                    img_num = match.group(1)
                    img_path = match.group(2).strip()
                    if os.path.exists(img_path):
                        image_paths.append(img_path)
                        return f'''<div class="image-container">
<img src="{img_path}" style="max-width:400px;cursor:pointer;">
<div class="image-path">[图片{img_num}: {img_path}]</div>
</div>'''
                    return match.group(0)
                return re.sub(pattern, replace_image, text), image_paths

            markdown_text, image_paths = _convert_image_paths(markdown_text)

            # 预处理：修正嵌套列表的缩进（避免无序子项被识别为有序项）
            # 一些编辑器/模型会在有序列表下的无序子项前仅缩进1-3个空格，
            # Python-Markdown 需至少4个空格才会解析为嵌套列表。
            def _normalize_markdown_lists(md: str) -> str:
                lines = md.splitlines()
                in_ol_block = False
                in_code_block = False
                fence_pat = re.compile(r"^\s*(```|~~~)")
                ol_pat = re.compile(r"^\s*\d+[\.)]\s")
                unordered_child_pat = re.compile(r"^ {1,3}[-*+]\s")

                for i, line in enumerate(lines):
                    # 代码块内不处理
                    if fence_pat.match(line):
                        in_code_block = not in_code_block
                        if in_code_block:
                            in_ol_block = False
                        continue
                    if in_code_block:
                        continue

                    # 空行或标题重置状态
                    if not line.strip() or line.lstrip().startswith('#'):
                        in_ol_block = False
                        continue

                    # 新的有序列表项开始
                    if ol_pat.match(line):
                        in_ol_block = True
                        continue

                    # 在有序列表块中，将1-3空格缩进的无序项提升为4空格
                    if in_ol_block and unordered_child_pat.match(line):
                        lines[i] = '    ' + line.lstrip()

                return '\n'.join(lines)

            markdown_text = _normalize_markdown_lists(markdown_text)

            # 配置markdown扩展
            md = markdown.Markdown(extensions=[
                'fenced_code',      # 代码块支持
                'tables',           # 表格支持  
                'nl2br',           # 换行转换
                'codehilite'       # 代码高亮
            ])
            
            # 转换markdown为HTML
            html_content = md.convert(markdown_text)
            
            # 添加CSS样式来美化HTML显示
            styled_html = f"""
            <style>
                body {{
                    font-family: 'Segoe UI', 'Arial', sans-serif;
                    line-height: 1.6;
                    color: #ffffff;
                    margin: 0;
                    padding: 5px;
                }}
                h1, h2, h3, h4, h5, h6 {{
                    color: #ffffff;
                    margin-top: 20px;
                    margin-bottom: 10px;
                }}
                h1 {{ border-bottom: 2px solid #444; padding-bottom: 8px; }}
                h2 {{ border-bottom: 1px solid #444; padding-bottom: 4px; }}
                code {{
                    background-color: #2d2d2d;
                    padding: 2px 4px;
                    border-radius: 3px;
                    font-family: 'Consolas', 'Monaco', monospace;
                    color: #ffa500;
                }}
                pre {{
                    background-color: #2d2d2d;
                    padding: 12px;
                    border-radius: 6px;
                    border-left: 4px solid #007bff;
                    overflow-x: auto;
                }}
                pre code {{
                    background-color: transparent;
                    padding: 0;
                    color: #ffffff;
                }}
                blockquote {{
                    border-left: 4px solid #007bff;
                    margin: 16px 0;
                    padding-left: 16px;
                    color: #cccccc;
                    font-style: italic;
                }}
                table {{
                    border-collapse: collapse;
                    width: 100%;
                    margin: 16px 0;
                }}
                th, td {{
                    border: 1px solid #444;
                    padding: 8px 12px;
                    text-align: left;
                }}
                th {{
                    background-color: #333;
                    font-weight: bold;
                }}
                ul, ol {{
                    padding-left: 20px;
                    margin: 12px 0;
                }}
                li {{
                    margin: 4px 0;
                }}
                a {{
                    color: #4da6ff;
                    text-decoration: none;
                }}
                a:hover {{
                    text-decoration: underline;
                }}
                .image-container {{
                    margin: 8px 0;
                }}
                .image-path {{
                    font-size: 11px;
                    color: #888;
                    margin-top: 4px;
                    word-break: break-all;
                }}
                img {{
                    border-radius: 4px;
                }}
            </style>
            {html_content}
            """

            # 先添加图片资源到文档（必须在setHtml之前）
            for path in image_paths:
                pixmap = QPixmap(path)
                if not pixmap.isNull():
                    self.document().addResource(QTextDocument.ResourceType.ImageResource, QUrl(path), pixmap)

            self.setHtml(styled_html)
            
        except Exception as e:
            # 如果markdown解析失败，回退到纯文本显示
            self.setPlainText(markdown_text)
            sys.stderr.write(f"Markdown parsing error: {e}\n")

    def mousePressEvent(self, event: QMouseEvent):
        """处理鼠标点击事件，支持点击图片放大和链接打开"""
        if event.button() == Qt.LeftButton:
            # 检查是否点击了链接
            anchor = self.anchorAt(event.pos())
            if anchor:
                QDesktopServices.openUrl(QUrl(anchor))
                return

            cursor = self.cursorForPosition(event.pos())
            char_format = cursor.charFormat()

            # 检查是否点击了图片
            if char_format.isImageFormat():
                img_path = char_format.toImageFormat().name()
                if os.path.exists(img_path):
                    dialog = ImageViewerDialog(img_path, self)
                    dialog.exec()
                    return

        super().mousePressEvent(event)

    def mouseMoveEvent(self, event: QMouseEvent):
        """处理鼠标移动事件，链接上显示手型光标"""
        anchor = self.anchorAt(event.pos())
        if anchor:
            self.viewport().setCursor(Qt.PointingHandCursor)
        else:
            self.viewport().setCursor(Qt.IBeamCursor)
        super().mouseMoveEvent(event)



"""
工作空间标签页 - 显示工作空间信息
"""
import os
import re
import subprocess
import platform
from typing import Optional
from functools import partial
from PySide6.QtWidgets import QVBoxLayout, QHBoxLayout, QWidget, QPushButton, QLabel, QMessageBox, QGridLayout, QTreeWidget, QTreeWidgetItem
from PySide6.QtCore import Qt
from PySide6.QtGui import QIcon, QBrush, QColor

try:
    from .base_tab import BaseTab
except ImportError:
    from base_tab import BaseTab

try:
    from ..components.markdown_display import MarkdownDisplayWidget
except ImportError:
    try:
        from components.markdown_display import MarkdownDisplayWidget
    except ImportError:
        from PySide6.QtWidgets import QTextEdit
        MarkdownDisplayWidget = QTextEdit

try:
    from ..workspace_manager import WorkspaceManager
except ImportError:
    try:
        from workspace_manager import WorkspaceManager
    except ImportError:
        WorkspaceManager = None


class WorkspaceTab(BaseTab):
    """工作空间标签页 - 显示工作空间详细信息"""

    def __init__(self, workspace_id: str, project_path: Optional[str] = None, parent=None):
        super().__init__(parent)
        self.workspace_id = workspace_id
        self.project_path = project_path
        self.workspace_config = None
        self.stage_template = None

        # 加载工作空间配置
        self._load_workspace_config()

        # 创建UI
        self.create_ui()

    def _load_workspace_config(self):
        """加载工作空间配置"""
        if not WorkspaceManager:
            return

        try:
            manager = WorkspaceManager(self.project_path)
            self.workspace_config = manager.load_workspace_config(self.workspace_id)

            # 加载阶段模板
            if self.workspace_config:
                stage_template_id = self.workspace_config.get('stage_template_id')
                if stage_template_id:
                    self.stage_template = manager.load_stage_template(stage_template_id)
        except Exception:
            # 静默处理加载失败
            self.workspace_config = None
            self.stage_template = None

    def create_ui(self):
        """创建工作空间标签页UI"""
        layout = QVBoxLayout(self)
        layout.setContentsMargins(10, 10, 10, 10)

        # 使用 MarkdownDisplayWidget 显示工作空间基本信息
        display_widget = MarkdownDisplayWidget()

        # 格式化工作空间信息为Markdown
        markdown_content = self._format_workspace_info()
        display_widget.setMarkdownText(markdown_content)

        layout.addWidget(display_widget)

        # 添加文件列表区域（如果有文件）
        files = self.workspace_config.get('files', []) if self.workspace_config else []
        if files:
            self._create_files_section(layout, files)

    def _format_workspace_info(self) -> str:
        """格式化工作空间信息为Markdown文本

        Returns:
            str: 格式化的Markdown文本
        """
        if not self.workspace_config:
            return "## ⚠️ 无法加载工作空间配置\n\n请检查工作空间ID是否正确。"

        parts = []

        # 1. 工作空间基本信息
        parts.append("## 📦 工作空间信息")
        parts.append("")
        parts.append(f"**ID:** `{self.workspace_id}`")

        goal = self.workspace_config.get('goal', '未设置')
        parts.append(f"**目标:** {goal}")

        status = self.workspace_config.get('status', '未知')
        parts.append(f"**状态:** {status}")

        created_at = self.workspace_config.get('created_at', '未知')
        parts.append(f"**创建时间:** {created_at}")

        updated_at = self.workspace_config.get('updated_at', '未知')
        parts.append(f"**更新时间:** {updated_at}")

        parts.append("")

        # 2. 阶段信息
        parts.append("## 📍 阶段信息")
        parts.append("")

        stage_template_id = self.workspace_config.get('stage_template_id', '未设置')
        parts.append(f"**模板:** `{stage_template_id}`")

        current_stage_id = self.workspace_config.get('current_stage_id', '未设置')
        parts.append(f"**当前阶段:** `{current_stage_id}`")

        # 显示当前阶段详细信息
        if self.stage_template and current_stage_id:
            workflow = self.stage_template.get('workflow', {})
            steps = workflow.get('steps', [])

            for step in steps:
                if step.get('id') == current_stage_id:
                    parts.append("")
                    parts.append(f"**阶段标题:** {step.get('title', '未知')}")
                    parts.append(f"**阶段描述:** {step.get('des', '无描述')}")
                    break

        parts.append("")

        # 3. 相关文档列表
        documents = self.workspace_config.get('documents', [])
        if documents:
            parts.append("## 📄 相关文档")
            parts.append("")
            for doc in documents:
                if isinstance(doc, dict):
                    title = doc.get('title', '未命名文档')
                    path = doc.get('path', '')
                    parts.append(f"- **{title}** (`{path}`)")
                else:
                    parts.append(f"- `{doc}`")
            parts.append("")

        # 注意：相关文件列表改为独立组件显示，不再在Markdown中显示

        return "\n".join(parts)

    def _create_files_section(self, layout, files: list):
        """创建文件列表显示区域（树形结构）

        Args:
            layout: 父布局
            files: 文件路径列表
        """
        # 导入配置管理
        try:
            from feedback_config import FeedbackConfig
        except ImportError:
            FeedbackConfig = None

        # 获取配置的IDE
        def get_configured_ide():
            """获取配置的IDE名称，优先级：配置文件 > 环境变量 > 默认值"""
            ide_name = None

            # 1. 尝试从配置文件读取
            if FeedbackConfig and self.project_path:
                try:
                    config_manager = FeedbackConfig(self.project_path)
                    ide_name = config_manager.get_ide()
                except Exception:
                    pass

            # 2. 如果配置文件没有，使用环境变量
            if not ide_name:
                ide_name = os.getenv('IDE')

            # 3. 最后使用默认值
            if not ide_name:
                ide_name = 'cursor'

            return ide_name

        # 文件去重并保持顺序
        unique_files = list(dict.fromkeys(files))

        # 路径规范化（移除tag前缀 + 相对路径转绝对路径）
        normalized_files = []
        for file_path in unique_files:
            # 移除路径开头的tag（如 Edit:, Create:, Read: 等）
            cleaned_path = re.sub(r'^[A-Za-z]+:', '', file_path)

            if not os.path.isabs(cleaned_path) and self.project_path:
                # 相对路径转绝对路径
                abs_path = os.path.join(self.project_path, cleaned_path)
                normalized_files.append(abs_path)
            else:
                normalized_files.append(cleaned_path)

        # 创建文件列表标题
        title_container = QWidget()
        title_layout = QHBoxLayout(title_container)
        title_layout.setContentsMargins(5, 10, 5, 5)
        title_layout.setSpacing(5)

        title_label = QLabel("📁 相关文件")
        title_label.setStyleSheet("""
            QLabel {
                font-size: 14px;
                font-weight: bold;
                color: #FFA500;
                padding: 5px 0;
            }
        """)
        title_layout.addWidget(title_label)
        title_layout.addStretch()

        layout.addWidget(title_container)

        # 找到所有文件的公共父目录
        common_prefix = self._find_common_prefix(normalized_files)

        # 创建文件树
        tree_widget = QTreeWidget()
        tree_widget.setHeaderHidden(True)
        tree_widget.setStyleSheet("""
            QTreeWidget {
                background: qlineargradient(x1:0, y1:0, x2:0, y2:1,
                    stop:0 rgba(76, 175, 80, 8),
                    stop:1 rgba(76, 175, 80, 12));
                border: 2px solid rgba(76, 175, 80, 35);
                border-radius: 8px;
                padding: 4px;
                font-size: 13px;
                outline: none;
                selection-background-color: transparent;
            }
            QTreeWidget::item {
                padding: 3px 6px;
                margin: 0px;
                border-radius: 4px;
                color: #2E7D32;
                min-height: 18px;
                selection-background-color: transparent;
            }
            QTreeWidget::item:hover {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                    stop:0 rgba(76, 175, 80, 25),
                    stop:1 rgba(129, 199, 132, 25));
                border-left: 3px solid #4CAF50;
                padding-left: 3px;
            }
            QTreeWidget::item:selected {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                    stop:0 rgba(76, 175, 80, 60),
                    stop:1 rgba(129, 199, 132, 60));
                border-left: 3px solid #66BB6A;
                padding-left: 3px;
                color: #FFFFFF;
                font-weight: 600;
                selection-background-color: transparent;
            }
            QTreeWidget::item:selected:!active {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                    stop:0 rgba(76, 175, 80, 60),
                    stop:1 rgba(129, 199, 132, 60));
                color: #FFFFFF;
                border-left: 3px solid #66BB6A;
                padding-left: 3px;
            }
            QTreeWidget::item:selected:hover {
                background: qlineargradient(x1:0, y1:0, x2:1, y2:0,
                    stop:0 rgba(76, 175, 80, 70),
                    stop:1 rgba(129, 199, 132, 70));
                border-left: 3px solid #81C784;
                color: #FFFFFF;
            }
            QTreeWidget::branch {
                background: transparent;
            }
            QTreeWidget::branch:has-children:!has-siblings:closed,
            QTreeWidget::branch:closed:has-children:has-siblings {
                border-image: none;
                image: url(none);
                margin: 2px;
            }
            QTreeWidget::branch:open:has-children:!has-siblings,
            QTreeWidget::branch:open:has-children:has-siblings {
                border-image: none;
                image: url(none);
                margin: 2px;
            }
        """)

        # 构建树形结构
        root_name = os.path.basename(common_prefix) if common_prefix else "Files"
        tree_root = {}  # 存储目录结构的字典树

        for file_path in normalized_files:
            # 获取相对路径
            if common_prefix:
                try:
                    rel_path = os.path.relpath(file_path, common_prefix)
                except ValueError:
                    # 如果无法获取相对路径（例如不同盘符），使用绝对路径
                    rel_path = file_path
            else:
                rel_path = file_path

            # 分割路径
            parts = rel_path.split(os.sep)

            # 构建树形结构
            current = tree_root
            for part in parts:
                if part not in current:
                    current[part] = {}
                current = current[part]

        # 递归创建树节点
        def create_tree_items(parent_item, tree_dict, current_path):
            """递归创建树节点

            Args:
                parent_item: 父节点（QTreeWidget或QTreeWidgetItem）
                tree_dict: 当前层级的字典树
                current_path: 当前路径
            """
            for name in sorted(tree_dict.keys()):
                full_path = os.path.join(current_path, name) if current_path else name
                abs_path = os.path.join(common_prefix, full_path) if common_prefix else full_path

                # 创建节点
                item = QTreeWidgetItem(parent_item)
                item.setText(0, name)

                # 判断是文件还是目录
                if tree_dict[name]:  # 有子节点，是目录
                    item.setIcon(0, self._get_folder_icon())
                    # 设置目录样式 - 更加突出
                    font = item.font(0)
                    font.setBold(True)
                    font.setPointSize(13)
                    item.setFont(0, font)
                    # 设置目录颜色为深绿色
                    item.setForeground(0, QBrush(QColor(27, 94, 32)))  # 深绿色
                    # 递归创建子节点
                    create_tree_items(item, tree_dict[name], full_path)
                else:  # 没有子节点，是文件
                    item.setIcon(0, self._get_file_icon())
                    # 设置文件颜色为中等绿色
                    item.setForeground(0, QBrush(QColor(56, 142, 60)))  # 中绿色
                    # 保存文件路径到item数据中
                    item.setData(0, Qt.UserRole, abs_path)
                    # 设置工具提示
                    ide_name = get_configured_ide()
                    ide_display_names = {
                        'cursor': 'Cursor',
                        'kiro': 'Kiro',
                        'vscode': 'VSCode',
                        'code': 'VSCode'
                    }
                    display_ide = ide_display_names.get(ide_name.lower(), ide_name)
                    item.setToolTip(0, f"双击在{display_ide}中打开: {abs_path}")

        # 创建根节点并添加所有文件
        create_tree_items(tree_widget, tree_root, "")

        # 展开所有节点
        tree_widget.expandAll()

        # 连接双击事件
        def on_item_double_clicked(item, column):
            """处理节点双击事件"""
            file_path = item.data(0, Qt.UserRole)
            if not file_path:  # 如果是目录节点，切换展开/折叠
                if item.isExpanded():
                    item.setExpanded(False)
                else:
                    item.setExpanded(True)
                return

            # 打开文件
            try:
                # 导入ide_utils模块
                import sys
                parent_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
                if parent_dir not in sys.path:
                    sys.path.insert(0, parent_dir)
                from ide_utils import open_project_with_ide

                # 获取IDE名称（使用配置）
                ide_name = get_configured_ide()

                # 使用通用的IDE打开函数
                success = open_project_with_ide(file_path, ide_name)

                if not success:
                    # 如果失败，使用系统默认编辑器打开
                    if platform.system() == "Darwin":
                        subprocess.run(["open", file_path], check=True)
                    elif platform.system() == "Windows":
                        os.startfile(file_path)
                    else:
                        subprocess.run(["xdg-open", file_path], check=True)

            except Exception as e:
                # 使用系统默认编辑器打开作为最终后备
                try:
                    if platform.system() == "Darwin":
                        subprocess.run(["open", file_path], check=True)
                    elif platform.system() == "Windows":
                        os.startfile(file_path)
                    else:
                        subprocess.run(["xdg-open", file_path], check=True)
                except Exception as e2:
                    file_name = os.path.basename(file_path)
                    QMessageBox.warning(self, "打开失败",
                        f"无法打开文件: {file_name}\n"
                        f"路径: {file_path}\n"
                        f"错误: {str(e2)}")

        tree_widget.itemDoubleClicked.connect(on_item_double_clicked)

        layout.addWidget(tree_widget)

    def _find_common_prefix(self, paths: list) -> str:
        """找到所有路径的公共父目录

        Args:
            paths: 文件路径列表

        Returns:
            str: 公共父目录路径
        """
        if not paths:
            return ""

        if len(paths) == 1:
            return os.path.dirname(paths[0])

        # 分割所有路径
        split_paths = [p.split(os.sep) for p in paths]

        # 找到最短路径的长度
        min_len = min(len(p) for p in split_paths)

        # 找到公共前缀
        common = []
        for i in range(min_len):
            parts = [p[i] for p in split_paths]
            if len(set(parts)) == 1:  # 所有路径在这一层都相同
                common.append(parts[0])
            else:
                break

        # 如果找到公共前缀，返回公共目录
        if common:
            return os.sep.join(common)

        # 如果没有公共前缀，返回空字符串
        return ""

    def _get_folder_icon(self) -> QIcon:
        """获取文件夹图标

        Returns:
            QIcon: 文件夹图标
        """
        # 使用系统提供的文件夹图标或自定义图标
        # 这里使用标准图标，也可以自定义
        from PySide6.QtWidgets import QStyle
        style = self.style()
        icon = style.standardIcon(QStyle.SP_DirIcon)
        return icon

    def _get_file_icon(self) -> QIcon:
        """获取文件图标

        Returns:
            QIcon: 文件图标
        """
        # 使用系统提供的文件图标或自定义图标
        from PySide6.QtWidgets import QStyle
        style = self.style()
        icon = style.standardIcon(QStyle.SP_FileIcon)
        return icon



#!/usr/bin/env python3
"""
会话状态管理器
用于管理stop hook和feedback UI之间的会话状态，避免死循环
"""
import json
import os
import sys
from datetime import datetime, timedelta
from pathlib import Path
from typing import Optional, Dict, Any

class SessionManager:
    """管理会话状态，避免stop hook死循环

    状态保存在 .workspace/chat_history/{session_id}.json 中
    """

    def __init__(self, session_id: Optional[str] = None, project_path: Optional[str] = None):
        """初始化会话管理器

        Args:
            session_id: 会话ID
            project_path: 项目路径，用于定位.workspace目录
        """
        self.session_id = session_id
        self.project_path = project_path or os.getcwd()

        # 不再加载所有sessions，改为按需加载单个session
        # self.sessions = self._load_sessions()
        # self._cleanup_old_sessions()

    def _get_chat_history_path(self, session_id: str) -> Path:
        """获取指定session的chat_history文件路径

        Args:
            session_id: 会话ID

        Returns:
            Path: chat_history文件路径
        """
        workspace_dir = Path(self.project_path) / '.workspace' / 'chat_history'
        workspace_dir.mkdir(parents=True, exist_ok=True)
        return workspace_dir / f"{session_id}.json"

    def _load_session_data(self, session_id: str) -> list:
        """加载指定session的chat_history数据（兼容旧格式）

        Args:
            session_id: 会话ID

        Returns:
            list: chat_history数据数组（内部格式，用于兼容现有代码）
        """
        chat_file = self._get_chat_history_path(session_id)
        if not chat_file.exists():
            return []

        try:
            with open(chat_file, 'r', encoding='utf-8') as f:
                data = json.load(f)

                # 新格式：{'dialogues': [...], 'control': {...}}
                if isinstance(data, dict) and 'dialogues' in data:
                    # 转换为内部数组格式
                    result = []

                    # 添加对话记录（自动添加type字段以兼容现有代码）
                    for dialogue in data.get('dialogues', []):
                        dialogue_with_type = dialogue.copy()
                        dialogue_with_type['type'] = 'dialogue'
                        result.append(dialogue_with_type)

                    # 添加control记录
                    if 'control' in data and data['control']:
                        result.append(data['control'])

                    return result

                # 旧格式：直接返回数组
                return data if isinstance(data, list) else []
        except (json.JSONDecodeError, IOError):
            return []

    def _save_session_data(self, session_id: str, data: list):
        """保存指定session的chat_history数据（新格式）

        Args:
            session_id: 会话ID
            data: chat_history数据数组
        """
        chat_file = self._get_chat_history_path(session_id)
        try:
            # 转换为新格式：dialogues数组
            new_format_data = {
                'dialogues': []
            }

            # 过滤出对话项
            for item in data:
                if isinstance(item, dict):
                    # 保留 agent 记录（直接添加）
                    if item.get('role') == 'agent':
                        new_format_data['dialogues'].append(item)
                        continue

                    item_type = item.get('type')
                    if item_type == 'dialogue':
                        # 创建新格式的对话项（移除type字段）
                        dialogue = {
                            'timestamp': item.get('timestamp', ''),
                            'time_display': item.get('time_display', ''),
                            'messages': item.get('messages', [])
                        }
                        new_format_data['dialogues'].append(dialogue)
                    elif item_type == 'stop_hook_status':
                        # 保留control字段（如果存在）
                        if 'control' not in new_format_data:
                            new_format_data['control'] = item

            with open(chat_file, 'w', encoding='utf-8') as f:
                json.dump(new_format_data, f, indent=2, ensure_ascii=False)
        except IOError as e:
            print(f"Error saving session data: {e}", file=sys.stderr)

    def _get_stop_hook_status(self, session_id: str) -> Optional[Dict[str, Any]]:
        """获取session的stop_hook状态

        Args:
            session_id: 会话ID

        Returns:
            Optional[Dict]: stop_hook状态数据，如果不存在则返回None
        """
        data = self._load_session_data(session_id)
        # 在数组末尾查找stop_hook_status元素
        for item in reversed(data):
            if isinstance(item, dict) and item.get('type') == 'stop_hook_status':
                return item
        return None

    def _set_stop_hook_status(self, session_id: str, status: str, action: Optional[str] = None):
        """设置session的stop_hook状态

        Args:
            session_id: 会话ID
            status: 状态值
            action: 动作描述
        """
        data = self._load_session_data(session_id)

        # 移除旧的stop_hook_status元素
        data = [item for item in data if not (isinstance(item, dict) and item.get('type') == 'stop_hook_status')]

        # 添加新的stop_hook_status元素
        data.append({
            'type': 'stop_hook_status',
            'status': status,
            'timestamp': datetime.now().isoformat(),
            'last_action': action or status
        })

        self._save_session_data(session_id, data)

    def _clear_stop_hook_status(self, session_id: str):
        """清除session的stop_hook状态

        Args:
            session_id: 会话ID
        """
        data = self._load_session_data(session_id)
        # 移除stop_hook_status元素
        data = [item for item in data if not (isinstance(item, dict) and item.get('type') == 'stop_hook_status')]
        self._save_session_data(session_id, data)

    def get_session_status(self, session_id: str) -> Optional[str]:
        """获取会话状态

        Args:
            session_id: 会话ID

        Returns:
            会话状态: "user_closed_by_button", "timeout_closed", "active" 或 None
        """
        status_data = self._get_stop_hook_status(session_id)
        if status_data:
            return status_data.get("status")
        return None

    def set_session_status(self, session_id: str, status: str, action: Optional[str] = None):
        """设置会话状态

        Args:
            session_id: 会话ID
            status: 状态 ("user_closed_by_button", "timeout_closed", "active")
            action: 最后的动作描述
        """
        self._set_stop_hook_status(session_id, status, action)

    def is_feedback_closed(self, session_id: str) -> bool:
        """检查会话是否因用户关闭feedback而结束

        Args:
            session_id: 会话ID

        Returns:
            True如果用户主动关闭了feedback窗口
        """
        status = self.get_session_status(session_id)
        return status == "user_closed"

    def mark_feedback_closed(self, session_id: str):
        """标记feedback被用户关闭（通用方法，保留向后兼容）

        Args:
            session_id: 会话ID
        """
        self.set_session_status(session_id, "user_closed", "feedback_window_closed_by_user")

    def mark_user_closed_by_button(self, session_id: str):
        """标记用户点击关闭按钮

        Args:
            session_id: 会话ID
        """
        self.set_session_status(session_id, "user_closed_by_button", "user_clicked_close_button")

    def mark_timeout_closed(self, session_id: str):
        """标记超时自动关闭

        Args:
            session_id: 会话ID
        """
        self.set_session_status(session_id, "timeout_closed", "feedback_timeout_auto_closed")

    def is_user_closed_by_button(self, session_id: str) -> bool:
        """检查是否用户点击关闭按钮

        Args:
            session_id: 会话ID

        Returns:
            True如果用户点击关闭按钮
        """
        status = self.get_session_status(session_id)
        return status == "user_closed_by_button"

    def is_timeout_closed(self, session_id: str) -> bool:
        """检查是否超时关闭

        Args:
            session_id: 会话ID

        Returns:
            True如果超时自动关闭
        """
        status = self.get_session_status(session_id)
        return status == "timeout_closed"

    def reset_on_feedback_show(self, session_id: str):
        """feedback展示时重置状态

        Args:
            session_id: 会话ID
        """
        # 清除所有状态
        self.clear_session(session_id)

    def get_block_count(self, session_id: str) -> int:
        """获取会话的阻止次数

        Args:
            session_id: 会话ID

        Returns:
            阻止次数
        """
        status_data = self._get_stop_hook_status(session_id)
        if status_data:
            return status_data.get("block_count", 0)
        return 0

    def increment_block_count(self, session_id: str) -> int:
        """增加会话的阻止次数

        Args:
            session_id: 会话ID

        Returns:
            更新后的阻止次数
        """
        status_data = self._get_stop_hook_status(session_id)
        if status_data:
            new_count = status_data.get("block_count", 0) + 1
            status_data["block_count"] = new_count
            status_data["timestamp"] = datetime.now().isoformat()

            # 更新到chat_history
            data = self._load_session_data(session_id)
            # 移除旧的stop_hook_status
            data = [item for item in data if not (isinstance(item, dict) and item.get('type') == 'stop_hook_status')]
            # 添加更新后的status
            data.append(status_data)
            self._save_session_data(session_id, data)
            return new_count
        else:
            # 首次创建
            self._set_stop_hook_status(session_id, "active", "block_count_increment")
            data = self._load_session_data(session_id)
            # 找到刚创建的status并设置block_count
            for item in reversed(data):
                if isinstance(item, dict) and item.get('type') == 'stop_hook_status':
                    item["block_count"] = 1
                    break
            self._save_session_data(session_id, data)
            return 1

    def clear_session(self, session_id: str):
        """清除会话状态

        Args:
            session_id: 会话ID
        """
        self._clear_stop_hook_status(session_id)


def main():
    """命令行接口"""
    import argparse
    
    parser = argparse.ArgumentParser(description='管理会话状态')
    parser.add_argument('action', choices=['get', 'set', 'check', 'mark_closed', 'clear'],
                        help='要执行的操作')
    parser.add_argument('session_id', help='会话ID')
    parser.add_argument('--status', help='设置的状态 (用于set操作)')
    parser.add_argument('--state-file', help='状态文件路径')
    
    args = parser.parse_args()
    
    manager = SessionManager(args.state_file)
    
    if args.action == 'get':
        status = manager.get_session_status(args.session_id)
        print(status if status else "none")
        
    elif args.action == 'set':
        if not args.status:
            print("Error: --status required for set action", file=sys.stderr)
            sys.exit(1)
        manager.set_session_status(args.session_id, args.status)
        print(f"Session {args.session_id} status set to {args.status}")
        
    elif args.action == 'check':
        is_closed = manager.is_feedback_closed(args.session_id)
        print("closed" if is_closed else "active")
        sys.exit(0 if not is_closed else 1)
        
    elif args.action == 'mark_closed':
        manager.mark_feedback_closed(args.session_id)
        print(f"Session {args.session_id} marked as closed by user")
        
    elif args.action == 'clear':
        manager.clear_session(args.session_id)
        print(f"Session {args.session_id} cleared")


if __name__ == "__main__":
    main()


"""
历史记录弹窗组件 - 聊天对话样式展示历史记录
"""

import logging
from typing import List, Dict, Any, Optional
from PySide6.QtWidgets import (
    QFrame, QVBoxLayout, QScrollArea, QWidget, QLabel, QPushButton, QHBoxLayout,
    QApplication, QGridLayout, QSizePolicy
)
from PySide6.QtCore import Qt, Signal, QPoint
from PySide6.QtGui import QKeyEvent, QFont, QTextCursor

# 设置日志
logging.basicConfig(level=logging.DEBUG)
logger = logging.getLogger(__name__)


class HistoryPopup(QFrame):
    """历史记录弹窗组件 - 聊天对话样式"""

    # 信号定义
    content_inserted = Signal(str)     # 内容插入信号
    content_copied = Signal(str)       # 内容复制信号
    popup_closed = Signal()            # 弹窗关闭信号

    def __init__(self, parent=None):
        super().__init__(parent)
        self.history_records = []  # 存储历史记录数据
        self.parent_window = None  # 父窗口引用，用于访问输入框

        self._setup_ui()
        self._setup_style()

        # 设置窗口属性
        self.setWindowFlags(Qt.Popup | Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_DeleteOnClose)

    def _setup_ui(self):
        """设置UI"""
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)

        # 标题栏
        title_frame = QFrame()
        title_frame.setStyleSheet("""
            QFrame {
                background: linear-gradient(90deg, #667eea 0%, #764ba2 100%);
                border-top-left-radius: 8px;
                border-top-right-radius: 8px;
                padding: 12px;
            }
        """)
        title_layout = QHBoxLayout(title_frame)
        title_layout.setContentsMargins(12, 8, 12, 8)

        self.title_label = QLabel("💬 聊天历史")
        self.title_label.setStyleSheet("""
            QLabel {
                color: white;
                font-size: 14px;
                font-weight: bold;
            }
        """)
        title_layout.addWidget(self.title_label)

        # 关闭按钮
        close_button = QPushButton("✕")
        close_button.setFixedSize(20, 20)
        close_button.setStyleSheet("""
            QPushButton {
                background: transparent;
                color: white;
                border: none;
                font-size: 14px;
                font-weight: bold;
            }
            QPushButton:hover {
                background: rgba(255, 255, 255, 0.2);
                border-radius: 10px;
            }
        """)
        close_button.clicked.connect(self.close)
        title_layout.addWidget(close_button)

        layout.addWidget(title_frame)

        # 聊天记录滚动容器
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.scroll_area.setStyleSheet("""
            QScrollArea {
                background: #1e1e1e;
                border: none;
            }
            QScrollBar:vertical {
                background: #2b2b2b;
                width: 8px;
                border-radius: 4px;
            }
            QScrollBar::handle:vertical {
                background: #555;
                border-radius: 4px;
                min-height: 30px;
            }
            QScrollBar::handle:vertical:hover {
                background: #666;
            }
        """)

        # 内容容器widget
        self.content_widget = QWidget()
        self.content_widget.setStyleSheet("background: #1e1e1e;")
        self.content_layout = QVBoxLayout(self.content_widget)
        self.content_layout.setSpacing(12)
        self.content_layout.setContentsMargins(12, 12, 12, 12)

        self.scroll_area.setWidget(self.content_widget)
        layout.addWidget(self.scroll_area)

        # 底部提示栏
        hint_frame = QFrame()
        hint_frame.setStyleSheet("""
            QFrame {
                background: #2b2b2b;
                border-bottom-left-radius: 8px;
                border-bottom-right-radius: 8px;
                padding: 8px;
            }
        """)
        hint_layout = QHBoxLayout(hint_frame)

        self.hint_label = QLabel("💡 点击消息可插入或复制 | ESC 关闭")
        self.hint_label.setStyleSheet("""
            QLabel {
                color: #888;
                font-size: 11px;
            }
        """)
        hint_layout.addWidget(self.hint_label, alignment=Qt.AlignCenter)

        layout.addWidget(hint_frame)

    def _setup_style(self):
        """设置样式"""
        self.setStyleSheet("""
            HistoryPopup {
                background: #1e1e1e;
                border: 1px solid #444;
                border-radius: 8px;
            }
        """)

    def set_history_records(self, records: List[Dict[str, Any]], parent_window=None):
        """设置历史记录列表

        Args:
            records: 历史记录列表，每个记录包含content, time_display等字段
            parent_window: 父窗口引用，用于访问输入框
        """
        self.history_records = records
        self.parent_window = parent_window
        self._update_history_list()

    def _update_history_list(self):
        """更新历史记录列表"""
        # 清空现有内容
        while self.content_layout.count():
            child = self.content_layout.takeAt(0)
            if child.widget():
                child.widget().deleteLater()

        if not self.history_records:
            # 显示空状态
            empty_label = QLabel("暂无聊天记录")
            empty_label.setAlignment(Qt.AlignCenter)
            empty_label.setStyleSheet("""
                QLabel {
                    color: #666;
                    padding: 40px;
                    font-size: 14px;
                }
            """)
            self.content_layout.addWidget(empty_label)
            return

        # 添加历史记录项（最新的在下面，模拟聊天界面）
        for record in self.history_records:
            self._create_dialogue_item(record)

        # 添加空白占位，让内容可以滚动到底部
        spacer = QWidget()
        spacer.setFixedHeight(20)
        self.content_layout.addWidget(spacer)

    def _create_dialogue_item(self, record: Dict[str, Any]):
        """创建对话记录项（兼容旧格式和新格式）

        Args:
            record: 记录数据
        """
        try:
            # 新格式：不含type字段，直接包含messages数组
            if 'messages' in record and isinstance(record.get('messages'), list):
                # 显示时间分隔线
                if record.get('time_display'):
                    time_label = QLabel(record['time_display'])
                    time_label.setAlignment(Qt.AlignCenter)
                    time_label.setStyleSheet("""
                        QLabel {
                            color: #666;
                            font-size: 10px;
                            padding: 4px;
                            margin: 8px 0;
                        }
                    """)
                    self.content_layout.addWidget(time_label)

                # 显示对话消息
                for msg in record['messages']:
                    self._create_message_bubble(msg)
            else:
                # 兼容旧格式的单条消息
                msg = {
                    'role': 'user',
                    'content': record.get('content', ''),
                    'time': record.get('time_display', '').split(' ')[-1] if 'time_display' in record else ''
                }
                self._create_message_bubble(msg)

        except Exception as e:
            logger.error(f"创建对话记录项失败: {e}")

    def _create_message_bubble(self, msg: Dict[str, Any]):
        """创建消息气泡

        Args:
            msg: 消息数据
        """
        role = msg.get('role', 'user')
        content = msg.get('content', '')
        time = msg.get('time', '')

        # 创建消息容器
        msg_container = QWidget()
        msg_layout = QHBoxLayout(msg_container)
        msg_layout.setContentsMargins(0, 4, 0, 4)
        msg_layout.setSpacing(10)
        msg_layout.setAlignment(Qt.AlignLeft)  # 统一左对齐

        # 头像 - 根据角色显示不同的图标和背景
        if role == 'user':
            avatar_label = QLabel("👤")
            avatar_bg = "linear-gradient(135deg, #667eea 0%, #764ba2 100%)"
        else:
            avatar_label = QLabel("🤖")
            avatar_bg = "linear-gradient(135deg, #00c853 0%, #00897b 100%)"

        avatar_label.setFixedSize(36, 36)
        avatar_label.setAlignment(Qt.AlignCenter)
        avatar_label.setStyleSheet(f"""
            QLabel {{
                background: {avatar_bg};
                border-radius: 18px;
                font-size: 18px;
            }}
        """)
        msg_layout.addWidget(avatar_label, alignment=Qt.AlignTop)

        # 消息内容区域
        content_frame = QFrame()
        # 移除最大宽度限制，让内容框架填充整个可用宽度
        content_frame.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        content_layout = QVBoxLayout(content_frame)
        content_layout.setContentsMargins(0, 0, 0, 0)
        content_layout.setSpacing(4)

        # 用户名称和时间行
        header_layout = QHBoxLayout()
        header_layout.setSpacing(8)

        # 用户名称
        name_label = QLabel("用户" if role == 'user' else "AI助手")
        name_label.setStyleSheet("""
            QLabel {
                color: #fff;
                font-size: 12px;
                font-weight: bold;
            }
        """)
        header_layout.addWidget(name_label)

        # 时间标签
        if time:
            time_label = QLabel(time)
            time_label.setStyleSheet("""
                QLabel {
                    color: #888;
                    font-size: 10px;
                }
            """)
            header_layout.addWidget(time_label)

        header_layout.addStretch()
        content_layout.addLayout(header_layout)

        # 消息内容框架
        bubble_frame = QFrame()
        bubble_frame.setStyleSheet("""
            QFrame {
                background: #2b2b2b;
                border: 1px solid #3a3a3a;
                border-radius: 8px;
                padding: 10px 12px;
            }
        """)

        bubble_layout = QVBoxLayout(bubble_frame)
        bubble_layout.setContentsMargins(0, 0, 0, 0)

        # 消息内容
        content_label = QLabel(content[:300] + "..." if len(content) > 300 else content)
        content_label.setWordWrap(True)
        content_label.setTextInteractionFlags(Qt.TextSelectableByMouse)
        content_label.setStyleSheet("""
            QLabel {
                color: #e0e0e0;
                font-size: 13px;
                line-height: 1.5;
            }
        """)
        if len(content) > 300:
            content_label.setToolTip(content)
        bubble_layout.addWidget(content_label)

        content_layout.addWidget(bubble_frame)

        # 添加操作按钮（悬浮显示）
        self._add_hover_buttons(bubble_frame, content)

        msg_layout.addWidget(content_frame, 1)  # 设置stretch factor为1，让内容框架占据所有可用空间

        self.content_layout.addWidget(msg_container)

    def _add_hover_buttons(self, bubble_frame, content):
        """添加悬浮操作按钮

        Args:
            bubble_frame: 消息气泡框架
            content: 消息内容
        """
        # 创建按钮容器
        button_container = QWidget(bubble_frame)
        button_container.setStyleSheet("""
            QWidget {
                background: rgba(0, 0, 0, 0.8);
                border-radius: 4px;
            }
        """)
        button_layout = QHBoxLayout(button_container)
        button_layout.setContentsMargins(4, 4, 4, 4)
        button_layout.setSpacing(4)

        # 插入按钮
        insert_btn = QPushButton("📥")
        insert_btn.setToolTip("插入到输入框")
        insert_btn.setFixedSize(24, 24)
        insert_btn.setStyleSheet("""
            QPushButton {
                background: transparent;
                color: white;
                border: none;
                font-size: 14px;
            }
            QPushButton:hover {
                background: rgba(255, 255, 255, 0.2);
                border-radius: 4px;
            }
        """)
        insert_btn.clicked.connect(lambda: self._handle_insert(content))
        button_layout.addWidget(insert_btn)

        # 复制按钮
        copy_btn = QPushButton("📋")
        copy_btn.setToolTip("复制到剪贴板")
        copy_btn.setFixedSize(24, 24)
        copy_btn.setStyleSheet("""
            QPushButton {
                background: transparent;
                color: white;
                border: none;
                font-size: 14px;
            }
            QPushButton:hover {
                background: rgba(255, 255, 255, 0.2);
                border-radius: 4px;
            }
        """)
        copy_btn.clicked.connect(lambda: self._handle_copy(content))
        button_layout.addWidget(copy_btn)

        # 初始隐藏，悬浮时显示
        button_container.hide()
        button_container.raise_()  # 确保按钮在最上层

        # 设置悬浮事件
        def show_buttons(event):
            # 计算按钮位置（右上角）
            button_container.move(
                bubble_frame.width() - button_container.width() - 8,
                8
            )
            button_container.show()

        def hide_buttons(event):
            button_container.hide()

        bubble_frame.enterEvent = show_buttons
        bubble_frame.leaveEvent = hide_buttons

    def _handle_insert(self, content: str):
        """处理插入操作"""
        try:
            # 尝试直接插入到输入框
            if self._insert_to_textbox(content):
                # 插入成功，发出信号并关闭弹窗
                self.content_inserted.emit(content)
                self.close()
            else:
                # 插入失败，回退到复制功能
                logger.warning("插入失败，回退到复制功能")
                self._handle_copy(content)
        except Exception as e:
            logger.error(f"插入内容失败: {e}")
            # 出错时回退到复制功能
            self._handle_copy(content)

    def _handle_copy(self, content: str):
        """处理复制操作"""
        try:
            clipboard = QApplication.clipboard()
            clipboard.setText(content)

            # 发出信号并关闭弹窗
            self.content_copied.emit(content)
            self.close()

        except Exception as e:
            logger.error(f"复制到剪贴板失败: {e}")

    def _insert_to_textbox(self, content: str) -> bool:
        """将内容插入到对话框输入框中

        Args:
            content: 要插入的内容

        Returns:
            bool: 插入是否成功
        """
        try:
            # 尝试获取主窗口的聊天标签页和输入框
            if self.parent_window:
                # 方法1：通过parent_window查找chat_tab
                if hasattr(self.parent_window, 'chat_tab') and self.parent_window.chat_tab:
                    feedback_text = self.parent_window.chat_tab.feedback_text
                    if feedback_text:
                        return self._do_insert(feedback_text, content)

                # 方法2：向上查找包含FeedbackTextEdit的窗口
                from PySide6.QtWidgets import QTextEdit
                parent = self.parent_window
                while parent:
                    # 在所有子控件中查找FeedbackTextEdit
                    feedback_widgets = parent.findChildren(QTextEdit)
                    for widget in feedback_widgets:
                        if hasattr(widget, 'pasted_images'):  # FeedbackTextEdit特有属性
                            return self._do_insert(widget, content)

                    parent = parent.parent()

            return False

        except Exception as e:
            logger.error(f"查找输入框失败: {e}")
            return False

    def _do_insert(self, text_widget, content: str) -> bool:
        """执行插入操作

        Args:
            text_widget: 文本输入控件
            content: 要插入的内容

        Returns:
            bool: 插入是否成功
        """
        try:
            # 获取当前文本
            current_text = text_widget.toPlainText()

            # 如果输入框有内容，在末尾添加换行后插入新内容
            if current_text.strip():
                new_text = current_text + "\n\n" + content
            else:
                new_text = content

            # 设置新文本
            text_widget.setPlainText(new_text)

            # 设置焦点到输入框
            text_widget.setFocus()

            # 将光标移动到末尾
            cursor = text_widget.textCursor()
            cursor.movePosition(QTextCursor.End)
            text_widget.setTextCursor(cursor)

            logger.info(f"成功插入内容: {content[:50]}...")
            return True

        except Exception as e:
            logger.error(f"执行插入操作失败: {e}")
            return False

    def keyPressEvent(self, event: QKeyEvent):
        """处理键盘事件"""
        if event.key() == Qt.Key_Escape:
            # ESC键关闭弹窗
            self.popup_closed.emit()
            self.close()
        else:
            super().keyPressEvent(event)

    def show_at_position(self, position: QPoint):
        """在指定位置显示弹窗"""
        # 获取屏幕信息
        screen = QApplication.primaryScreen()
        screen_geometry = screen.availableGeometry()

        # 设置弹窗大小与feedback UI一致：500x700
        popup_width = 500
        popup_height = 700

        # 设置弹窗大小
        self.resize(popup_width, popup_height)

        # 使用传入的位置（左边缘对齐）
        if position and position.x() >= 0 and position.y() >= 0:
            x = position.x()  # 使用传入的x坐标（已经是左对齐）
            y = position.y()  # 使用传入的y坐标

            # 确保不超出屏幕右边界
            if x + popup_width > screen_geometry.right():
                x = screen_geometry.right() - popup_width - 10

            # 确保不超出屏幕底部
            if y + popup_height > screen_geometry.bottom():
                y = screen_geometry.bottom() - popup_height - 10

            # 确保不超出屏幕左边界和顶部
            if x < screen_geometry.x():
                x = screen_geometry.x()
            if y < screen_geometry.y():
                y = screen_geometry.y()
        else:
            # 居中显示
            x = screen_geometry.x() + (screen_geometry.width() - popup_width) // 2
            y = screen_geometry.y() + (screen_geometry.height() - popup_height) // 2

        self.move(x, y)
        self.show()
        self.setFocus()

        # 滚动到底部显示最新消息
        self.scroll_area.verticalScrollBar().setValue(
            self.scroll_area.verticalScrollBar().maximum()
        )

        logger.info(f"历史记录弹窗显示在位置: ({x}, {y}), 大小: {popup_width}x{popup_height}")


"""
聊天历史记录管理模块

功能：
1. 保存用户输入的聊天内容
2. 加载和显示历史记录
3. 提供弹窗方式显示历史记录
4. 支持插入和复制功能
"""

import os
import json
from datetime import datetime
from typing import List, Dict, Optional
from PySide6.QtWidgets import QApplication, QMessageBox
from PySide6.QtCore import QPoint

# 导入历史记录弹窗组件
try:
    from .history_popup import HistoryPopup
    HISTORY_POPUP_AVAILABLE = True
except ImportError:
    try:
        from history_popup import HistoryPopup
        HISTORY_POPUP_AVAILABLE = True
    except ImportError:
        HISTORY_POPUP_AVAILABLE = False
        print("Warning: HistoryPopup component not available")


class ChatHistoryManager:
    """聊天历史记录管理器"""
    
    def __init__(self, project_path: Optional[str] = None, session_id: Optional[str] = None):
        self.project_path = project_path
        self.session_id = session_id
    
    def get_history_file_path(self) -> Optional[str]:
        """获取历史记录文件路径"""
        # 如果没有session_id,返回None
        if not self.session_id:
            return None

        if self.project_path:
            return os.path.join(self.project_path, '.workspace', 'chat_history', f'{self.session_id}.json')
        else:
            # 如果没有项目路径，使用脚本目录
            script_dir = os.path.dirname(os.path.abspath(__file__))
            return os.path.join(script_dir, '..', '.workspace', 'chat_history', f'{self.session_id}.json')
    
    def save_to_history(self, content: str) -> bool:
        """保存内容到历史记录

        Args:
            content: 要保存的内容

        Returns:
            bool: 保存是否成功
        """
        if not content.strip():
            return False

        try:
            # 获取历史记录文件路径
            history_file = self.get_history_file_path()

            # 如果没有session_id,静默跳过
            if not history_file:
                return False

            # 读取完整的文件数据
            file_data = {}
            if os.path.exists(history_file):
                with open(history_file, 'r', encoding='utf-8') as f:
                    data = json.load(f)

                    # 新格式：{'dialogues': [...], 'control': {...}}
                    if isinstance(data, dict) and 'dialogues' in data:
                        file_data = data
                    # 旧格式数组，转换为新格式
                    elif isinstance(data, list):
                        file_data = {
                            'dialogues': [record for record in data if isinstance(record, dict) and record.get('type') != 'stop_hook_status']
                        }
                    else:
                        file_data = {'dialogues': []}
            else:
                # 文件不存在，初始化为新格式
                file_data = {'dialogues': []}

            # 添加新记录到dialogues数组
            new_record = {
                'content': content.strip(),
                'timestamp': datetime.now().isoformat(),
                'time_display': datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            }

            # 确保dialogues字段存在
            if 'dialogues' not in file_data:
                file_data['dialogues'] = []

            file_data['dialogues'].append(new_record)

            # 保存完整的文件数据
            os.makedirs(os.path.dirname(history_file), exist_ok=True)
            with open(history_file, 'w', encoding='utf-8') as f:
                json.dump(file_data, f, ensure_ascii=False, indent=2)

            return True

        except Exception as e:
            print(f"保存历史记录失败: {e}")
            return False
    
    def load_history_from_file(self) -> List[Dict]:
        """从文件加载历史记录（兼容旧格式和新格式）"""
        try:
            history_file = self.get_history_file_path()

            # 如果没有session_id,返回空列表
            if not history_file:
                return []

            if os.path.exists(history_file):
                with open(history_file, 'r', encoding='utf-8') as f:
                    data = json.load(f)

                    # 新格式：{'dialogues': [...], 'control': {...}}
                    if isinstance(data, dict) and 'dialogues' in data:
                        # 返回dialogues数组
                        return data.get('dialogues', [])

                    # 旧格式数组
                    if isinstance(data, list):
                        # 过滤掉control记录，保留对话记录
                        return [record for record in data if isinstance(record, dict) and record.get('type') != 'stop_hook_status']

                    return []
            return []
        except Exception as e:
            print(f"加载历史记录失败: {e}")
            return []
    
    def get_recent_history(self, count: Optional[int] = None) -> List[Dict]:
        """获取最近的历史记录

        Args:
            count: 获取记录数量，如果为None则返回所有记录

        Returns:
            List[Dict]: 历史记录列表
        """
        history = self.load_history_from_file()
        if count is None:
            return history  # 返回所有历史记录
        return history[-count:]

    def get_agent_records_after_last_user(self) -> List[Dict]:
        """获取最后一条用户消息之后的所有 agent 记录"""
        history = self.load_history_from_file()

        # 找到最后一条用户消息的索引
        last_user_idx = -1
        for i, record in enumerate(history):
            if record.get('role') == 'user' or 'messages' in record:
                last_user_idx = i

        # 返回之后的所有 agent 记录
        if last_user_idx >= 0:
            return [r for r in history[last_user_idx + 1:] if r.get('role') == 'agent']

        # 如果没有用户消息，返回所有 agent 记录
        return [r for r in history if r.get('role') == 'agent']

    def _get_draft_file_path(self) -> Optional[str]:
        """获取草稿文件路径（独立文件，避免被其他代码覆盖）"""
        if not self.project_path or not self.session_id:
            return None
        return os.path.join(self.project_path, '.workspace', 'chat_history', f'{self.session_id}_draft.json')

    def get_latest_draft(self) -> Optional[dict]:
        """获取最近的未提交草稿"""
        try:
            draft_file = self._get_draft_file_path()
            if not draft_file or not os.path.exists(draft_file):
                return None

            with open(draft_file, 'r', encoding='utf-8') as f:
                return json.load(f)
        except Exception:
            return None

    def save_draft(self, text: str) -> bool:
        """保存草稿到独立文件"""
        try:
            draft_file = self._get_draft_file_path()
            if not draft_file:
                return False

            os.makedirs(os.path.dirname(draft_file), exist_ok=True)
            with open(draft_file, 'w', encoding='utf-8') as f:
                json.dump({'text': text}, f, ensure_ascii=False)

            return True
        except Exception:
            return False

    def clear_draft(self) -> bool:
        """删除草稿文件"""
        try:
            draft_file = self._get_draft_file_path()
            if draft_file and os.path.exists(draft_file):
                os.remove(draft_file)
            return True
        except Exception:
            return False
    
    def show_history_dialog(self, parent=None) -> None:
        """显示历史记录弹窗
        
        Args:
            parent: 父窗口
        """
        try:
            # 加载历史记录
            history = self.get_recent_history()
            
            if not history:
                QMessageBox.information(parent, "历史记录", "暂无历史记录")
                return
            
            # 检查弹窗组件是否可用
            if not HISTORY_POPUP_AVAILABLE:
                QMessageBox.critical(parent, "错误", "历史记录弹窗组件不可用")
                return
            
            # 显示历史记录弹窗
            self._show_history_popup(parent, history)
            
        except Exception as e:
            print(f"显示历史记录失败: {e}")
            QMessageBox.critical(parent, "错误", f"显示历史记录失败: {str(e)}")
    
    def _show_history_popup(self, parent, history: List[Dict]):
        """显示历史记录弹窗"""
        try:
            # 创建弹窗
            popup = HistoryPopup(parent)
            
            # 设置历史记录数据
            popup.set_history_records(history, parent)
            
            # 连接信号
            popup.content_inserted.connect(lambda content: print(f"✅ 内容已插入: {content[:50]}..."))
            popup.content_copied.connect(lambda content: print(f"✅ 内容已复制: {content[:50]}..."))
            popup.popup_closed.connect(lambda: print("📝 历史记录弹窗已关闭"))
            
            # 计算弹窗位置（在历史按钮附近）
            if hasattr(parent, 'history_button'):
                button = parent.history_button
                # 获取按钮的全局位置
                button_pos = button.mapToGlobal(button.rect().bottomLeft())
                # 稍微偏移一下位置
                popup_pos = QPoint(button_pos.x() - 200, button_pos.y() + 5)
            else:
                # 如果找不到按钮，在父窗口中央显示
                if parent:
                    parent_rect = parent.geometry()
                    popup_pos = QPoint(
                        parent_rect.x() + parent_rect.width() // 2 - 250,
                        parent_rect.y() + parent_rect.height() // 2 - 200
                    )
                else:
                    popup_pos = QPoint(100, 100)
            
            # 显示弹窗
            popup.show_at_position(popup_pos)
            
        except Exception as e:
            print(f"显示历史记录弹窗失败: {e}")
            QMessageBox.critical(parent, "错误", f"显示历史记录弹窗失败: {str(e)}") 


#!/usr/bin/env python3
import os
import requests
import datetime
import logging
from pathlib import Path
from dotenv import load_dotenv

# 导入统一日志系统
from debug_logger import get_debug_logger

# 导入路径配置模块
try:
    from path_config import get_path_config
    PATH_CONFIG_AVAILABLE = True
except ImportError:
    PATH_CONFIG_AVAILABLE = False

# 全局变量
_project_path = None
_env_loaded = False

def setup_stats_logger():
    """设置统计日志，统一保存到脚本所在目录"""
    logger = logging.getLogger('statistics')
    if not logger.handlers:
        logger.setLevel(logging.DEBUG)
        
        # 统一使用脚本所在目录，确保日志位置固定
        script_dir = Path(__file__).parent
        log_file = script_dir / "log.txt"
        
        file_handler = logging.FileHandler(log_file, mode='a', encoding='utf-8')
        file_handler.setLevel(logging.DEBUG)
        
        formatter = logging.Formatter(
            '%(asctime)s [%(levelname)s] %(name)s: %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S'
        )
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)
    
    return logger

# 创建全局统计日志实例
stats_logger = setup_stats_logger()

def get_absolute_path(path: str, base_dir: str) -> str:
    """将路径转换为绝对路径
    如果是绝对路径直接返回，否则基于base_dir转换为绝对路径
    """
    if not path:
        return path
    return path if os.path.isabs(path) else os.path.join(base_dir, path)

def set_project_path(project_path: str):
    """设置项目路径并加载对应的.env文件"""
    global _project_path, _env_loaded
    _project_path = project_path
    
    if not project_path:
        logger = get_debug_logger()
        logger.log_warning("统计模块：未提供项目路径", "STATS")
        return
    
    if PATH_CONFIG_AVAILABLE:
        # 使用path_config获取.env文件路径
        path_config = get_path_config(project_path)
        env_path = path_config.get_env_file_path()
        
        if env_path and os.path.exists(env_path):
            load_dotenv(env_path)
            _env_loaded = True
        else:
            logger = get_debug_logger()
            logger.log_warning(f"统计模块未找到.env文件: {env_path}", "STATS")
    else:
        # 兜底方案：直接拼接路径
        env_path = os.path.join(project_path, '.agent', '.env')
        if os.path.exists(env_path):
            load_dotenv(env_path)
            _env_loaded = True
        else:
            logger = get_debug_logger()
            logger.log_warning(f"统计模块兜底也未找到.env文件: {env_path}", "STATS")

def get_user_info():
    """从GitLab认证文件获取用户信息 - 功能已移除

    Returns:
        Tuple[str, str]: (user_id, user_name)
    """
    # GitLab认证功能已移除
    logger = get_debug_logger()
    logger.log_info("GitLab认证功能已移除", "STATS")
    return None, None

def report_action(data):
    """向API上报操作数据

    Args:
        data (dict): 上报数据对象，包含以下字段：
            - user_name (str): 用户名(必填)
            - action (str): 操作类型，如'执行工作流'、'工作流:下一步'等(必填)
            - content (str, optional): 操作内容详情，默认为空字符串
            - workflow_name (str, optional): 工作流名称，默认为空字符串
            - task_name (str, optional): 任务名称，默认为空字符串
            - step_name (str, optional): 步骤名称，默认为空字符串
            - task_id (str, optional): 任务ID，默认为空字符串
            其他字段会作为扩展字段一起上报

    Returns:
        bool: 上报是否成功

    Raises:
        ValueError: 当必填字段缺失时抛出异常
    """
    if not data.get('action'):
        raise ValueError("action 是必填字段")

    # 获取当前时间戳(毫秒)
    current_time = int(datetime.datetime.now().timestamp() * 1000)

    # 定义所有必要字段及其默认值
    user_id, user_name = get_user_info()
    required_fields = {
        'user_name': user_name,  # 必填，但已经在上面检查过了
        'action': '',     # 必填，但已经在上面检查过了
        'content': '',
        'workflow_name': '',
        'task_name': '',
        'step_name': '',
        'task_id': '',
        'time': current_time
    }

    # 使用默认值填充缺失字段
    for field, default_value in required_fields.items():
        data.setdefault(field, default_value)

    try:
        stats_logger.info(f"开始上报统计: action={data['action']}, user={user_name}")
        response = requests.post('https://gitstat.aqwhr.cn/tapd/user-action-data/save', json=data)
        stats_logger.info(f"统计上报成功: status={response.status_code}, action={data['action']}")
        return response.status_code == 200
    except Exception as e:
        stats_logger.error(f"统计上报失败: {e}")
        return False

if __name__ == '__main__':
    # 测试用户信息获取
    user_id, user_name = get_user_info()
    print(f"User ID: {user_id}, User Name: {user_name}")

    # 测试数据上报
    if user_name:
        # 测试基本上报
        success = report_action({
            'user_name': user_name,
            'action': '测试上报',
            'content': '测试内容'
        })
        print(f"Basic report result: {'Success' if success else 'Failed'}")

        # 测试完整参数上报
        success = report_action({
            'user_name': user_name,
            'action': '工作流执行',
            'content': '执行开发工作流',
            'workflow_name': '开发工作流',
            'task_name': '代码开发',
            'step_name': '编写功能',
            'task_id': '1'
        })
        print(f"Full report result: {'Success' if success else 'Failed'}")

        # 测试带扩展字段的上报
        success = report_action({
            'user_name': user_name,
            'action': '自定义事件',
            'content': '测试扩展字段',
            'custom_field': '自定义值',
            'extra_info': {
                'key1': 'value1',
                'key2': 'value2'
            }
        })
        print(f"Extended report result: {'Success' if success else 'Failed'}")



"""
会话列表UI模块入口
支持: python -m src-min.ui (本地开发) 或 python -m ui (PyPI安装)
"""
import sys
from PySide6.QtWidgets import QApplication
from .session_list_ui import SessionListUI

def main():
    app = QApplication(sys.argv)
    window = SessionListUI()
    window.show()
    sys.exit(app.exec())

if __name__ == "__main__":
    main()
